package org.objectweb.jinha.actions;

import java.util.List;

import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreateRequest;
import org.eclipse.gef.requests.SimpleFactory;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.ui.IWorkbenchPart;
import org.objectweb.jinha.editor.Editor;
import org.objectweb.jinha.editparts.BusinessMethodListEditPart;
import org.objectweb.jinha.editparts.DiagramElementEditPart;
import org.objectweb.jinha.editparts.EntityEditPart;
import org.objectweb.jinha.editparts.MessageDrivenEditPart;
import org.objectweb.jinha.editparts.SessionEditPart;
import org.objectweb.jinha.model.BusinessMethodModel;
import org.objectweb.jinha.model.ModelCreationFactory;



 
/**
 * this class is used for creating businessMethod in a contextual ejb menu
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class CreateBusinessMethodAction extends SelectionAction {

    /**
     *  <code>ID</code> the id of action
     */
    public static final String ID = "create business method action";

    /**
     * @param part
     */
    public CreateBusinessMethodAction(IWorkbenchPart part) {
        super(part);
    }


    /** 
     * 
     * initialize the action
     * 
     * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#init()
     * 
     */
    public void init() {
        setId(ID);
        setText("create business method");
    }

    /**
     * set the enabled state of action to true or false
     *
     * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
     * @return the state of the action
     */
    protected boolean calculateEnabled() {
        List list = getSelectedObjects();
        if (list.size() == 1) {
            Object obj = list.get(0);
            if (obj instanceof DiagramElementEditPart) {
                return true;
            }
        }
        return false;
    }

    /** 
     * launch the action of creating business method
     * @see org.eclipse.jface.action.IAction#run()
     * 
     */
    public void run() {
        System.out.println("action CreateBusinessMethodAction performed");
        List list = getSelectedObjects();
        if (list.size() == 1) {
            Object obj = list.get(0);
            if (obj instanceof SessionEditPart) {
                SessionEditPart part = (SessionEditPart) obj;
                BusinessMethodListEditPart methpart = part
                        .getBusinessMethodListEditPart();
                CreateRequest req = new CreateRequest();
                req.setType(RequestConstants.REQ_CREATE);
                req.setFactory(new ModelCreationFactory(BusinessMethodModel.class,Editor.getModelManager().getResource()));
                Command command = methpart.getCommand(req);
                if (command != null && getCommandStack() != null)
                    getCommandStack().execute(command);
            }
            else if (obj instanceof EntityEditPart) {
                EntityEditPart part = (EntityEditPart) obj;
                BusinessMethodListEditPart methpart = part
                        .getBusinessMethodListEditPart();
                CreateRequest req = new CreateRequest();
                req.setType(RequestConstants.REQ_CREATE);
                req.setFactory(new ModelCreationFactory(BusinessMethodModel.class,Editor.getModelManager().getResource()));
                Command command = methpart.getCommand(req);
                if (command != null && getCommandStack() != null)
                   getCommandStack().execute(command);
            }
            else if (obj instanceof MessageDrivenEditPart) {
                MessageDrivenEditPart part = (MessageDrivenEditPart) obj;
                BusinessMethodListEditPart methpart = part
                        .getBusinessMethodListEditPart();
                CreateRequest req = new CreateRequest();
                req.setType(RequestConstants.REQ_CREATE);
                req.setFactory(new ModelCreationFactory(BusinessMethodModel.class,Editor.getModelManager().getResource()));
                Command command = methpart.getCommand(req);
                if (command != null && getCommandStack() != null)
                    getCommandStack().execute(command);
            }
        }
    }

}