
package org.objectweb.jinha.actions;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.FileEditorInput;
import org.objectweb.jinha.editor.Editor;

import com.objectlearn.jdt.j2ee.core.Project;


/**
 * this class is used for launching JINHa
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class LaunchJINHaAction implements IActionDelegate,
        IObjectActionDelegate {

	

    /**
     * <code>page</code> the current Workbench
     */
    private IWorkbenchPage page;

    /**
     * <code>selectedFolder</code> the ejbModule selected in the workspace
     */
    private IFolder        selectedFolder;
    private IPackageFragment packageFragment;


    /**
     * @param action
     * @param part the part to activate
     * @see org.eclipse.ui.IObjectActionDelegate#setActivePart(org.eclipse.jface.action.IAction, org.eclipse.ui.IWorkbenchPart)
     */
    public void setActivePart(IAction action, IWorkbenchPart part) {
        page = part.getSite().getPage();

        
    }

    /**
     * called when the selection in workspace has changed 
     * @param action
     * @param selection the selected items in workspace
     * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
     */
    public void selectionChanged(IAction action, ISelection selection) {
        selectedFolder = null;
        packageFragment=null;
        if (selection instanceof IStructuredSelection) {
            IStructuredSelection ssel = (IStructuredSelection) selection;
            Object o = ssel.getFirstElement();
            if (isEjbModule(o)) {
                if(o instanceof IFolder) {
                    selectedFolder = (IFolder) ssel.getFirstElement();
                action.setEnabled(selectedFolder != null);
                }
                else {
                    packageFragment=(IPackageFragment)ssel.getFirstElement();
                    action.setEnabled(packageFragment != null);
                }
            }
            else action.setEnabled(false);
        }
        

    }

    /**
     * @param o the object to be tested
     * @return true if o is an ejbModule, eslse return false
     */
    public boolean isEjbModule(Object o) {
       
        IProject p=null;
        String nameModule=null;
        if (o instanceof IFolder) {
           p= ((IFolder)o).getProject();
           nameModule=((IFolder)o).getName();
        }
        else if (o instanceof IPackageFragment) {
            nameModule=((IPackageFragment)o).getElementName().substring(((IPackageFragment)o).getElementName().lastIndexOf(".")+1);
            p= ((IPackageFragment)o).getJavaProject().getProject();
        }
        Project lombozProject = null;
        lombozProject = lombozProject.findProject(p);
        
        if(lombozProject.findModule(nameModule)!=null)
            return true;
        else return false;

        
           
    
    
    }

    /** 
     * launch the action of opening JINHa editor
     * @see org.eclipse.jface.action.IAction#run()
     * 
     */
    public void run(IAction action) {


        IPath p = null;
        if (selectedFolder!=null)
            p=selectedFolder.getFullPath();
        else if(packageFragment!=null)
            p=packageFragment.getPath();
        else return;
        
        IFile file = ResourcesPlugin.getWorkspace().getRoot().getFile(p);
        FileEditorInput fileEditorInput = new FileEditorInput(file);
        Editor editor = null;
        try {
                        editor = (Editor) page.openEditor(fileEditorInput,
                    "org.objectweb.jinha");
            
            if (selectedFolder!=null) {
                editor.setFileInput(file);
                editor.getContents().setModuleName(selectedFolder.getName());
                editor.getContents().setProjectName(selectedFolder.getProject().getName());
                editor.getContents().getModuleEMF().setName(selectedFolder.getProject().getName()+"."+selectedFolder.getName());
            }
               
            else {
            	editor.setFileInput(file);
                editor.getContents().setModuleName(packageFragment.getElementName());
                editor.getContents().setProjectName(packageFragment.getJavaProject().getElementName());
                editor.getContents().getModuleEMF().setName(packageFragment.getJavaProject().getElementName()+"."+packageFragment.getElementName());
            }
            editor.setInitialized(true);
            editor.setTabName(editor.getContents().getProjectName()+"."+editor.getContents().getModuleName());
           
        }
        catch (PartInitException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }
}

