package org.objectweb.jinha.commands;

import java.util.Iterator;

import org.eclipse.gef.commands.Command;
import org.objectweb.jinha.model.AttributeModel;
import org.objectweb.jinha.model.CMPModel;
import org.objectweb.jinha.model.ListModel;
import org.objectweb.jinha.model.emf.jinha.GraphicalEntity;
import org.objectweb.jinha.model.emf.jinha.GraphicalMessageDriven;
import org.objectweb.jinha.model.emf.jinha.GraphicalSession;

/**
 * this command create an attribute in a SessionModel, EntityModel or MessageDrivenModel and in the EMF model associated
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class CreateAttributeCommand extends MainCommand {

    /**
     * <code>list</code> the list of attributes
     */
    private ListModel      list;

    /**
     * <code>newAttribute</code> the attribute to be added to the list
     */
    private AttributeModel newAttribute;

   
    /**
     * @param list the list wher the attribute is added
     * @param newAttribute the attribute to be added
     */
    public CreateAttributeCommand(ListModel list, AttributeModel newAttribute) {
        this.newAttribute = newAttribute;
       
        this.list = list;
        
    }


    /**
     * execute the command which add the attribute to the list of attributes
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute() {
    	String ident=getIdent();
        this.newAttribute.setSignature("private","int",ident);
        this.newAttribute.getAttribute().getName().setValue(ident);
        this.newAttribute.getAttribute().getType().setValue("int");
        this.newAttribute.getAttribute().getVisibility().setValue("private");
        list.addElement(newAttribute);

        if (list.getEjb() instanceof GraphicalSession)
            ((GraphicalSession) list.getEjb()).getSessionBean()
                    .getJavaAttributes().add(newAttribute.getAttribute());
         else if(list.getEjb() instanceof GraphicalEntity)
            ((GraphicalEntity)list.getEjb()).getEntityBean().getJavaAttributes().add(newAttribute.getAttribute());
        // else if(list.getEjb() instanceof GraphicalMessageDriven)
        //     ((GraphicalMessageDriven)list.getEjb()).getMessageDrivenBean().getJavaAttributes().add(newAttribute.getAttribute());

    }

    /**
	 * @return
	 */
	private String getIdent() {
		String id="untitledAttribute"+getCpt();
		Iterator it=list.getElementList().iterator();
		while(it.hasNext()){
			if(id.equals(((AttributeModel)it.next()).getIdentifier())){
				id+="_"+getCpt();
				it=list.getElementList().iterator();
			}
		}
		return id;
	}
    
    /**
     * redo the execute() method
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo() {
        execute();
//        res.getContents().add(newAttribute.getAttribute());
    }

    /**
     * undo the operations performed by the execute() method
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo() {
        list.removeElement(newAttribute.getAttribute());
 //       res.getContents().remove(newAttribute);
        if (list.getEjb() instanceof GraphicalSession)
            ((GraphicalSession) list.getEjb()).getSessionBean()
                    .getJavaAttributes().remove(newAttribute.getAttribute());
              else if(list.getEjb() instanceof GraphicalEntity)
            ((GraphicalEntity)list.getEjb()).getEntityBean().getJavaAttributes().add(newAttribute.getAttribute());
        // else if(list.getEjb() instanceof GraphicalMessageDriven)
        //     ((GraphicalMessageDriven)list.getEjb()).getMessageDrivenBean().getJavaAttributes().add(newAttribute.getAttribute());

    }
    /**
     * @return Returns the cpt.
     */
    public  int getCpt() {
        if(list.getEjb() instanceof GraphicalSession) {
            return ((GraphicalSession)list.getEjb()).getSessionBean().getJavaAttributes().size()+1;
        }
        else if(list.getEjb() instanceof GraphicalEntity) {
            return ((GraphicalEntity)list.getEjb()).getEntityBean().getJavaAttributes().size()+1;
            
        }
        return 1;
    }
}