package org.objectweb.jinha.commands;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.commands.Command;
import org.objectweb.jinha.model.DiagramElementModel;
import org.objectweb.jinha.model.DiagramModel;
import org.objectweb.jinha.model.EntityModel;
import org.objectweb.jinha.model.MessageDrivenModel;
import org.objectweb.jinha.model.SessionModel;
import org.objectweb.jinha.model.emf.jinha.GraphicalSession;

/**
 * add an ejb to the diagramModel
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class CreateEjbCommand extends MainCommand {

    private DiagramModel        diagram;

    private DiagramElementModel newClass;

    private Point               location;

    private Dimension           size;

    

    /**
     * @param diagram the diagram which will have the new ejb
     * @param newClass the ejb to be added
     * @param location the point where will be displayed the ejb in the diagram
     * @param size the size of the ejb
     */
    public CreateEjbCommand(DiagramModel diagram, DiagramElementModel newClass,
            Point location, Dimension size) {
        this.newClass = newClass;
        this.location = location;
        this.size = size;
        this.diagram = diagram;
    }

    /**
     * performs the adding the the ejb to the diagram
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute() {
        res.getContents().add(newClass.getGraphicalEjb());
        newClass.setLocation(location);
        if (size != null)
            newClass.setSize(size);
        diagram.addElement(newClass);
        if (newClass instanceof SessionModel) {
            diagram.getModuleEMF().getGraphicalSessionBeans().add(
                    newClass.getGraphicalEjb());
        }
        else if (newClass instanceof EntityModel)
            diagram.getModuleEMF().getGraphicalEntityBeans().add(
                    newClass.getGraphicalEjb());
        else if (newClass instanceof MessageDrivenModel)
            diagram.getModuleEMF().getGraphicalMessageDrivenBeans().add(
                    newClass.getGraphicalEjb());

    }

    /**
     * performs execute()
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo() {
        diagram.addElement(newClass);
        res.getContents().add(newClass.getGraphicalEjb());
        if (newClass instanceof SessionModel)
            diagram.getModuleEMF().getGraphicalSessionBeans().add(
                    newClass.getGraphicalEjb());
        else if (newClass instanceof EntityModel)
            diagram.getModuleEMF().getGraphicalEntityBeans().add(
                    newClass.getGraphicalEjb());
        else if (newClass instanceof MessageDrivenModel)
            diagram.getModuleEMF().getGraphicalMessageDrivenBeans().add(
                    newClass.getGraphicalEjb());

    }

    /**
     * remove the ejb of the diagram
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo() {
        diagram.removeElement(newClass);
        res.getContents().remove(newClass.getGraphicalEjb());
        if (newClass instanceof SessionModel)
            diagram.getModuleEMF().getGraphicalSessionBeans().remove(
                    newClass.getGraphicalEjb());
        else if (newClass instanceof EntityModel)
            diagram.getModuleEMF().getGraphicalEntityBeans().remove(
                    newClass.getGraphicalEjb());
        else if (newClass instanceof MessageDrivenModel)
            diagram.getModuleEMF().getGraphicalMessageDrivenBeans().remove(
                    newClass.getGraphicalEjb());

    }

}