package org.objectweb.jinha.commands;

import java.util.ArrayList;

import org.eclipse.gef.commands.Command;
import org.objectweb.jinha.model.CMRModel;
import org.objectweb.jinha.model.EntityModel;
import org.objectweb.jinha.model.RelationModel;

/**
 * create and add a relation between to entity beans
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class CreateRelationCommand extends MainCommand {

    private EntityModel   source;

    private EntityModel   target;

    private RelationModel relation;

    CMRModel              CMRsrc;

    /**
     * @return Returns the relation.
     */
    public RelationModel getRelation() {
        return relation;
    }

    /**
     * @param relation
     *            The relation to set.
     */
    public void setRelation(RelationModel connection) {
        this.relation = connection;
    }

    /**
     * @return Returns the source.
     */
    public EntityModel getSource() {
        return source;
    }

    /**
     * @param source
     *            The source to set.
     */
    public void setSource(EntityModel source) {
        this.source = source;
    }

    /**
     * @return Returns the target.
     */
    public EntityModel getTarget() {
        return target;
    }

    /**
     * @param target
     *            The target to set.
     */
    public void setTarget(EntityModel target) {
        this.target = target;
    }

    /**
     * @param source the ejb source
     * @param target the ejb target
     */
    public CreateRelationCommand(EntityModel source, EntityModel target) {
        this.source = source;
        this.target = target;


    }

    /**
     * performs the creation of the relation between 2 entity beans 
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute() {
        relation = new RelationModel(res);

        if (CMRsrc == null)
            CMRsrc = new CMRModel(res);
        CMRsrc.setRelation(relation);
        relation.setCMRsrc(CMRsrc);
        relation.setCMRtarget(null);
        relation.setSource(source);
        relation.setTarget(target);
        CMRsrc.setSignature(CMRModel.PUBLIC, target.getName(), "get"+target.getName(), new ArrayList());
        
        source.getCMRs().addElement(CMRsrc);

    }

    /**
     * same as execute()
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo() {
        CMRsrc.setRelation(relation);

        relation.setSource(source);
        relation.setTarget(target);
        source.getCMRs().addElement(CMRsrc);

        relation.setCMRsrc(CMRsrc);
        relation.setCMRtarget(null);

    }

    /**
     * remove the relation between the 2 entity beans
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo() {
        CMRsrc.setRelation(null);

        source.getCMRs().removeElement(CMRsrc);

        relation.setSource(null);
        relation.setTarget(null);
        relation.setCMRsrc(null);
        relation.setCMRtarget(null);

    }

    /**
     * @return Returns the cMRsrc.
     */
    public CMRModel getCMRsrc() {
        return CMRsrc;
    }

    /**
     * @param rsrc
     *            The cMRsrc to set.
     */
    public void setCMRsrc(CMRModel rsrc) {
        CMRsrc = rsrc;
    }
}