package org.objectweb.jinha.commands;

import org.eclipse.draw2d.geometry.Rectangle;
import org.objectweb.jinha.model.DiagramElementModel;


/**
 * change the constraint (position and size) of a diagram element (session, entity or message driven bean)
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class DiagramElementChangeConstraintCommand extends MainCommand {

    private DiagramElementModel element;

    private Rectangle           oldConstraint;

    private Rectangle           newConstraint;

    /**
     * @param element the ejb which you want change the constraint
     * @param newConstraint the constraint to be applied to the ejb
     */
    public DiagramElementChangeConstraintCommand(DiagramElementModel element,
            Rectangle newConstraint) {
        this.newConstraint = newConstraint;
        this.element = element;
    }

    /**
     * changes the constraint of the ejb
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute() {
        oldConstraint = element.getConstraint();
        element.setConstraint(newConstraint);
        element.getGraphicalEjb().setHeight(newConstraint.height);
        element.getGraphicalEjb().setWidth(newConstraint.width);
        element.getGraphicalEjb().setX(newConstraint.x);
        element.getGraphicalEjb().setY(newConstraint.y);
    }

    /**
     * same as execute()
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo() {
        element.setConstraint(newConstraint);
        element.getGraphicalEjb().setHeight(newConstraint.height);
        element.getGraphicalEjb().setWidth(newConstraint.width);
        element.getGraphicalEjb().setX(newConstraint.x);
        element.getGraphicalEjb().setY(newConstraint.y);
    }

    /**
     * restore the previous constraint
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo() {
        element.setConstraint(oldConstraint);
        element.getGraphicalEjb().setHeight(oldConstraint.height);
        element.getGraphicalEjb().setWidth(oldConstraint.width);
        element.getGraphicalEjb().setX(oldConstraint.x);
        element.getGraphicalEjb().setY(oldConstraint.y);
    }
}