package org.objectweb.jinha.commands;

import org.objectweb.jinha.model.DiagramElementModel;
import org.objectweb.jinha.model.DiagramModel;
import org.objectweb.jinha.model.EntityModel;
import org.objectweb.jinha.model.MessageDrivenModel;
import org.objectweb.jinha.model.SessionModel;
import org.objectweb.jinha.model.emf.jinha.GraphicalEntity;
import org.objectweb.jinha.model.emf.jinha.GraphicalMessageDriven;
import org.objectweb.jinha.model.emf.jinha.GraphicalSession;

/**
 * this class delete an ejb from a diagram
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class DiagramElementDeleteCommand extends MainCommand {

    private DiagramElementModel child;

    private DiagramModel        diagram;

    /**
     * @param diagram the diagram where will be deleted 
     * @param child the ejb to be deleted
     */
    public DiagramElementDeleteCommand(DiagramModel diagram,
            DiagramElementModel child) {
        this.child = child;
        this.diagram = diagram;
    }

    /**
     * performs the deletion of the ejb from the diagram
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute() {
        diagram.removeElement(child);
        res.getContents().remove(child.getGraphicalEjb());
        res.getContents().remove(child.getGraphicalEjb());
        if(child instanceof SessionModel)
            diagram.getModuleEMF().getGraphicalSessionBeans().remove(child.getGraphicalEjb());
        else if(child instanceof EntityModel)
            diagram.getModuleEMF().getGraphicalEntityBeans().remove(child.getGraphicalEjb());
        else if(child instanceof MessageDrivenModel)
            diagram.getModuleEMF().getGraphicalMessageDrivenBeans().remove(child.getGraphicalEjb());
        
        
    }

    /**
     * same as execute()
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo() {
        execute();
        res.getContents().remove(child.getGraphicalEjb());
    }

    /**
     * restore the ejb in the diagram
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo() {
        diagram.addElement(child);
        res.getContents().add(child.getGraphicalEjb());
        if(child instanceof SessionModel)
            diagram.getModuleEMF().getGraphicalSessionBeans().add(child);
        else if(child instanceof EntityModel)
            diagram.getModuleEMF().getGraphicalEntityBeans().add(child);
        else if(child instanceof MessageDrivenModel)
            diagram.getModuleEMF().getGraphicalMessageDrivenBeans().add(child);

    }
}