package org.objectweb.jinha.commands;

import org.objectweb.jinha.model.AttributeModel;
import org.objectweb.jinha.model.BusinessMethodModel;
import org.objectweb.jinha.model.CMPModel;
import org.objectweb.jinha.model.HomeMethodModel;
import org.objectweb.jinha.model.ListModel;
import org.objectweb.jinha.model.emf.jinha.GraphicalEntity;
import org.objectweb.jinha.model.emf.jinha.GraphicalSession;

/**
 * remove an orphan element from aan ejb's list
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class ListElementOrphanCommand extends MainCommand {

    private Object    child;

    private ListModel list;

    private int       oldIndex;

    /**
     * @param list the list where will be deleted the orphan element
     * @param child the orphan element to be deleted
     */
    public ListElementOrphanCommand(ListModel list, Object child) {
        this.child = child;
        this.list = list;
    }

    /**
     * performs the deletion
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute() {
        oldIndex = list.getIndexOf(child);
        list.removeElement(child);
        if(list.getEjb() instanceof GraphicalSession) {
            if(child instanceof AttributeModel)
                ((GraphicalSession)list.getEjb()).getSessionBean().getJavaAttributes().remove(((AttributeModel)child).getAttribute());
            else if (child instanceof HomeMethodModel)
                ((GraphicalSession)list.getEjb()).getSessionBean().getMethods().remove(((HomeMethodModel)child).getMethod());
            else if (child instanceof BusinessMethodModel)
                ((GraphicalSession)list.getEjb()).getSessionBean().getMethods().remove(((BusinessMethodModel)child).getMethod());

        }
        else if(list.getEjb() instanceof GraphicalEntity) {
            if(child instanceof AttributeModel)
                ((GraphicalEntity)list.getEjb()).getEntityBean().getJavaAttributes().remove(((AttributeModel)child).getAttribute());
            else if (child instanceof HomeMethodModel)
                ((GraphicalEntity)list.getEjb()).getEntityBean().getMethods().remove(((HomeMethodModel)child).getMethod());
            else if (child instanceof BusinessMethodModel)
                ((GraphicalEntity)list.getEjb()).getEntityBean().getMethods().remove(((BusinessMethodModel)child).getMethod());
            else if (child instanceof CMPModel)
                ((GraphicalEntity)list.getEjb()).getEntityBean().getFieldMappings().remove(((CMPModel)child).getFieldMapping());

        }
        /*
        else if(list.getEjb() instanceof GraphicalMessageDriven) {
            if(child instanceof AttributeModel)
                ((GraphicalMessageDriven)list.getEjb()).getMessageDrivenBean().getJavaAttributes().remove(((AttributeModel)child).getAttribute());
            else if (child instanceof HomeMethodModel)
                ((GraphicalMessageDriven)list.getEjb()).getMessageDrivenBean().getMethods().remove(((HomeMethodModel)child).getMethod());
            else if (child instanceof BusinessMethodModel)
                ((GraphicalMessageDriven)list.getEjb()).getMessageDrivenBean().getMethods().remove(((BusinessMethodModel)child).getMethod());

        }*/
    }

    /**
     * same as execute();
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo() {
        list.removeElement(child);
        if(list.getEjb() instanceof GraphicalSession) {
            if(child instanceof AttributeModel)
                ((GraphicalSession)list.getEjb()).getSessionBean().getJavaAttributes().remove(((AttributeModel)child).getAttribute());
            else if (child instanceof HomeMethodModel)
                ((GraphicalSession)list.getEjb()).getSessionBean().getMethods().remove(((HomeMethodModel)child).getMethod());
            else if (child instanceof BusinessMethodModel)
                ((GraphicalSession)list.getEjb()).getSessionBean().getMethods().remove(((BusinessMethodModel)child).getMethod());

        }
        else if(list.getEjb() instanceof GraphicalEntity) {
            if(child instanceof AttributeModel)
                ((GraphicalEntity)list.getEjb()).getEntityBean().getJavaAttributes().remove(((AttributeModel)child).getAttribute());
            else if (child instanceof HomeMethodModel)
                ((GraphicalEntity)list.getEjb()).getEntityBean().getMethods().remove(((HomeMethodModel)child).getMethod());
            else if (child instanceof BusinessMethodModel)
                ((GraphicalEntity)list.getEjb()).getEntityBean().getMethods().remove(((BusinessMethodModel)child).getMethod());
            else if (child instanceof CMPModel)
                ((GraphicalEntity)list.getEjb()).getEntityBean().getFieldMappings().remove(((CMPModel)child).getFieldMapping());

        }
        /*
        else if(list.getEjb() instanceof GraphicalMessageDriven) {
            if(child instanceof AttributeModel)
                ((GraphicalMessageDriven)list.getEjb()).getMessageDrivenBean().getJavaAttributes().remove(((AttributeModel)child).getAttribute());
            else if (child instanceof HomeMethodModel)
                ((GraphicalMessageDriven)list.getEjb()).getMessageDrivenBean().getMethods().remove(((HomeMethodModel)child).getMethod());
            else if (child instanceof BusinessMethodModel)
                ((GraphicalMessageDriven)list.getEjb()).getMessageDrivenBean().getMethods().remove(((BusinessMethodModel)child).getMethod());

        }*/
    }

    /**
     * restore the element in the list
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo() {
        list.addElement(child, oldIndex);
        if(list.getEjb() instanceof GraphicalSession) {
            if(child instanceof AttributeModel)
                ((GraphicalSession)list.getEjb()).getSessionBean().getJavaAttributes().add(((AttributeModel)child).getAttribute());
            else if (child instanceof HomeMethodModel)
                ((GraphicalSession)list.getEjb()).getSessionBean().getMethods().add(((HomeMethodModel)child).getMethod());
            else if (child instanceof BusinessMethodModel)
                ((GraphicalSession)list.getEjb()).getSessionBean().getMethods().add(((BusinessMethodModel)child).getMethod());

        }
        else if(list.getEjb() instanceof GraphicalEntity) {
            if(child instanceof AttributeModel)
                ((GraphicalEntity)list.getEjb()).getEntityBean().getJavaAttributes().add(((AttributeModel)child).getAttribute());
            else if (child instanceof HomeMethodModel)
                ((GraphicalEntity)list.getEjb()).getEntityBean().getMethods().add(((HomeMethodModel)child).getMethod());
            else if (child instanceof BusinessMethodModel)
                ((GraphicalEntity)list.getEjb()).getEntityBean().getMethods().add(((BusinessMethodModel)child).getMethod());
            else if (child instanceof CMPModel)
                ((GraphicalEntity)list.getEjb()).getEntityBean().getFieldMappings().add(((CMPModel)child).getFieldMapping());

        }
        /*
        else if(list.getEjb() instanceof GraphicalMessageDriven) {
            if(child instanceof AttributeModel)
                ((GraphicalMessageDriven)list.getEjb()).getMessageDrivenBean().getJavaAttributes().add(((AttributeModel)child).getAttribute());
            else if (child instanceof HomeMethodModel)
                ((GraphicalMessageDriven)list.getEjb()).getMessageDrivenBean().getMethods().add(((HomeMethodModel)child).getMethod());
            else if (child instanceof BusinessMethodModel)
                ((GraphicalMessageDriven)list.getEjb()).getMessageDrivenBean().getMethods().add(((BusinessMethodModel)child).getMethod());

        }*/
    }
}