package org.objectweb.jinha.commands;


import org.eclipse.draw2d.AbsoluteBendpoint;
import org.eclipse.draw2d.geometry.Point;
import org.objectweb.jinha.model.RelationModel;

/**
 * change the location of a bendpoint 
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class MoveBendpointCommand extends MainCommand {

    private Point             newLocation;

    private Point             oldLocation;

    private int               index;

    private RelationModel     model;

    private AbsoluteBendpoint bendpoint;

    /**
     * @param model the relation which contains the bendpoint
     * @param index the index of the bendpoint in the list of bendpoints
     * @param newLocation the new location
     */
    public MoveBendpointCommand(RelationModel model, int index,
            Point newLocation) {
        this.model = model;
        this.index = index;
        this.newLocation = newLocation;
    }

    /**
     * performs the change of constraint on the bendpoint
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute() {
        AbsoluteBendpoint bendpoint = (AbsoluteBendpoint) model
                .getBendpoint(index);
        oldLocation = bendpoint.getLocation();
        bendpoint = new AbsoluteBendpoint(newLocation);
        model.replaceBendpoint(index, bendpoint);
    }

    /**
     * same as execute()
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo() {
        bendpoint = new AbsoluteBendpoint(newLocation);
        model.replaceBendpoint(index, bendpoint);
    }

    /**
     * restore the old constraint to the bendpoint
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo() {
        bendpoint = new AbsoluteBendpoint(oldLocation);
        model.replaceBendpoint(index, bendpoint);
    }
}