package org.objectweb.jinha.editor;
 
import java.util.EventObject;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.commands.CommandStackListener;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.views.contentoutline.ContentOutlinePage;
import org.objectweb.jinha.editparts.AttributeEditPart;
import org.objectweb.jinha.editparts.BusinessMethodEditPart;
import org.objectweb.jinha.editparts.CMPEditPart;
import org.objectweb.jinha.editparts.CMREditPart;
import org.objectweb.jinha.editparts.FinderEditPart;
import org.objectweb.jinha.editparts.HomeMethodEditPart;
import org.objectweb.jinha.figures.AttributeFigure;
import org.objectweb.jinha.figures.EjbFigureBorder;
import org.objectweb.jinha.figures.MethodFigure;
import org.objectweb.jinha.figures.CMPFigure;
import org.objectweb.jinha.model.AttributeListModel;
import org.objectweb.jinha.model.AttributeModel;
import org.objectweb.jinha.model.BusinessMethodListModel;
import org.objectweb.jinha.model.BusinessMethodModel;
import org.objectweb.jinha.model.CMPListModel;
import org.objectweb.jinha.model.CMPModel;
import org.objectweb.jinha.model.CMRListModel;
import org.objectweb.jinha.model.CMRModel;
import org.objectweb.jinha.model.EntityModel;
import org.objectweb.jinha.model.FinderListModel;
import org.objectweb.jinha.model.FinderModel;
import org.objectweb.jinha.model.HomeMethodListModel;
import org.objectweb.jinha.model.HomeMethodModel;
import org.objectweb.jinha.model.MessageDrivenModel;
import org.objectweb.jinha.model.SessionModel;

/**
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 */

public class DiagramOutlinePage extends ContentOutlinePage implements
        CommandStackListener {

    private EditPart       input;

    private ActionRegistry ar;

    private CommandStack   commandStack;

    /**
     * @param commandStack
     * @param input
     * @param ar
     */
    public DiagramOutlinePage(CommandStack commandStack, EditPart input,
            ActionRegistry ar) {
        this.commandStack = commandStack;
        commandStack.addCommandStackListener(this);
        this.input = input;
        this.ar = ar;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.part.IPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl(Composite parent) {
        super.createControl(parent);

        TreeViewer treeViewer = getTreeViewer();
        treeViewer.setContentProvider(new MyTreeContentProvider2());
        treeViewer.setLabelProvider(new MyLabelProvider2());
        treeViewer.setInput(input);
        treeViewer.setAutoExpandLevel(TreeViewer.ALL_LEVELS);

        ContextMenuProviderImpl menuManager = new ContextMenuProviderImpl(ar,
                null);
        getControl().setMenu(menuManager.createContextMenu(getControl()));
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.commands.CommandStackListener#commandStackChanged(java.util.EventObject)
     */
    public void commandStackChanged(EventObject event) {
        //mthode de barbarre. yahaaaaaaaaaaaaaa !!!
        getTreeViewer().refresh();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.part.IPage#dispose()
     */
    public void dispose() {
        super.dispose();
        commandStack.removeCommandStackListener(this);
    }
}

class MyTreeContentProvider2 implements ITreeContentProvider {

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
     */
    public Object[] getChildren(Object parentElement) {
        if (parentElement instanceof EditPart)
            return ((EditPart) parentElement).getChildren().toArray();
        else
            return new Object[0];
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
     */
    public Object getParent(Object element) {
        if (element instanceof EditPart)
            return ((EditPart) element).getParent();
        else
            return new Object[0];
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
     */
    public boolean hasChildren(Object element) {
        return !(element instanceof BusinessMethodEditPart
                || element instanceof AttributeEditPart
                || element instanceof HomeMethodEditPart
                || element instanceof CMPEditPart
                || element instanceof CMREditPart || element instanceof FinderEditPart);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
     */
    public Object[] getElements(Object inputElement) {
        return getChildren(inputElement);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IContentProvider#dispose()
     */
    public void dispose() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
     *      java.lang.Object, java.lang.Object)
     */
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
    }
}

class MyLabelProvider2 implements ILabelProvider {

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.ILabelProvider#getImage(java.lang.Object)
     */
    public Image getImage(Object element) {
        if (element instanceof EditPart) {
            Object model = ((EditPart) element).getModel();
            if (model instanceof BusinessMethodModel)
                return MethodFigure.getVisibilityIcon(((BusinessMethodModel) model)
                                .getVisibility());
            else if (model instanceof HomeMethodModel)
                return MethodFigure.getVisibilityIcon(((HomeMethodModel) model)
                        .getVisibility());
            else if (model instanceof CMPModel)
                return CMPFigure.getPKicon(((CMPModel) model)
                        .isPrimaryKey());
            else if (model instanceof CMRModel)
                return MethodFigure.getVisibilityIcon(((CMRModel) model)
                        .getVisibility());
            else if (model instanceof FinderModel)
                return MethodFigure.getVisibilityIcon(((FinderModel) model)
                        .getVisibility());
            else if (model instanceof AttributeModel)
                return AttributeFigure
                        .getVisibilityIcon(((AttributeModel) model)
                                .getVisibility());
            else if (model instanceof SessionModel)
                return EjbFigureBorder.IMAGE;
            else if (model instanceof EntityModel)
                return EjbFigureBorder.IMAGE;
            else if (model instanceof MessageDrivenModel)
                return EjbFigureBorder.IMAGE;
        }
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
     */
    public String getText(Object element) {
        if (element instanceof EditPart) {
            Object model = ((EditPart) element).getModel();
            if (model instanceof BusinessMethodModel)
                return model.toString();
            else if (model instanceof HomeMethodModel)
                return model.toString();
            else if (model instanceof CMPModel)
                return model.toString();
            else if (model instanceof CMRModel)
                return model.toString();
            else if (model instanceof FinderModel)
                return model.toString();
            else if (model instanceof AttributeModel)
                return model.toString();
            else if (model instanceof SessionModel)
                //return ((SessionModel) model).getName();
                return ((SessionModel) model).getPropertyEjbBeanSession()
                        .getEjbBeanName();
            else if (model instanceof EntityModel)
                return ((EntityModel) model).getPropertyEjbBeanEntity()
                        .getEjbBeanName();
            else if (model instanceof MessageDrivenModel)
                return ((MessageDrivenModel) model).getName();
            else if (model instanceof BusinessMethodListModel)
                return "Business:";
            else if (model instanceof AttributeListModel)
                return "attributes:";
            else if (model instanceof HomeMethodListModel)
                return "Homes:";
            else if (model instanceof CMPListModel)
                return "CMPs:";
            else if (model instanceof CMRListModel)
                return "CMRs:";
            else if (model instanceof FinderListModel)
                return "Finders:";
        }
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IBaseLabelProvider#addListener(org.eclipse.jface.viewers.ILabelProviderListener)
     */
    public void addListener(ILabelProviderListener listener) {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IBaseLabelProvider#dispose()
     */
    public void dispose() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IBaseLabelProvider#isLabelProperty(java.lang.Object,
     *      java.lang.String)
     */
    public boolean isLabelProperty(Object element, String property) {
        return false;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IBaseLabelProvider#removeListener(org.eclipse.jface.viewers.ILabelProviderListener)
     */
    public void removeListener(ILabelProviderListener listener) {
    }
}