package org.objectweb.jinha.editor;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.util.URI;
import org.eclipse.gef.DefaultEditDomain;
import org.eclipse.gef.GEFPlugin;
import org.eclipse.gef.KeyHandler;
import org.eclipse.gef.KeyStroke;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.editparts.FreeformGraphicalRootEditPart;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.gef.ui.parts.GraphicalEditorWithPalette;
import org.eclipse.gef.ui.parts.GraphicalViewerKeyHandler;
import org.eclipse.jface.action.IAction;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.PropertySheetPage;
import org.objectweb.jinha.actions.CreateAttributeAction;
import org.objectweb.jinha.actions.CreateBusinessMethodAction;
import org.objectweb.jinha.actions.CreateCMPAction;
import org.objectweb.jinha.actions.CreateFinderAction;
import org.objectweb.jinha.actions.CreateHomeMethodAction;
import org.objectweb.jinha.commands.MainCommand;
import org.objectweb.jinha.model.DiagramModel;
import org.objectweb.jinha.model.EMFModelManager;

public class Editor extends GraphicalEditorWithPalette
{
	private PropertySheetPage propPage;
	private DiagramModel content=null;
    private static EMFModelManager modelManager=null;
    private boolean isInitialized=false;
    private IFile fileInput=null;

    

	public Editor()
	{
       
		setEditDomain(new DefaultEditDomain(this));
         
       
	}
    

    protected PaletteRoot getPaletteRoot() {
        
        content = new DiagramModel();

        modelManager =new EMFModelManager();
        modelManager.setEditor(this);
        
        URI file = URI.createFileURI("saveJinha.xmi");
        modelManager.createModule(file);
        content.setModuleEMF(modelManager.getModel());
        MainCommand.setRes(modelManager.getResource());

        
        
        PaletteRoot palette = null;
        
        
        GlobalPalette globalPalette=new GlobalPalette(modelManager.getResource());
        palette = globalPalette.createPalette();
        return palette;
        
    } 

	protected void configureGraphicalViewer()
	{
		//root editpart:
		FreeformGraphicalRootEditPart root = new FreeformGraphicalRootEditPart();
		getGraphicalViewer().setRootEditPart(root);
		//menu contextuel:
		getGraphicalViewer().setContextMenu(new ContextMenuProviderImpl(getActionRegistry(), getGraphicalViewer()));

		//keyhandler:
		
		KeyHandler keyHandler = new KeyHandler();
		keyHandler.put(KeyStroke.getPressed(SWT.DEL, 127, 0), getActionRegistry().getAction(GEFActionConstants.DELETE));

		keyHandler.put(KeyStroke.getPressed('a', 0), getActionRegistry().getAction(CreateAttributeAction.ID));
		keyHandler.put(KeyStroke.getPressed('m', 0), getActionRegistry().getAction(CreateBusinessMethodAction.ID));
		getGraphicalViewer().setKeyHandler(new GraphicalViewerKeyHandler(getGraphicalViewer()).setParent(keyHandler));
	}
	protected void initializeGraphicalViewer()
	{
		getGraphicalViewer().setEditPartFactory(new EditPartFactoryImpl());
		getGraphicalViewer().setContents(getContents());
		setTabName(getContents().getProjectName()+"/"+getContents().getModuleName());
		modelManager.setPage(this.getEditorSite().getPage());
	}
	public DiagramModel getContents()
	{
		return content;
	}

    public void setTabName(String title) {
        this.setPartName(title);
    }
	public void createActions()
	{
		super.createActions();

		ActionRegistry registry = getActionRegistry();
		IAction action;

		action = new CreateAttributeAction((IWorkbenchPart) this);
		registry.registerAction(action);
		getSelectionActions().add(action.getId());

		action = new CreateBusinessMethodAction((IWorkbenchPart) this);
		registry.registerAction(action);
		getSelectionActions().add(action.getId());
        
        action = new CreateHomeMethodAction((IWorkbenchPart) this);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());
        
        action = new CreateCMPAction((IWorkbenchPart) this);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());
        
 
        
        action = new CreateFinderAction((IWorkbenchPart) this);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());

	}
	public void createPartControl(Composite parent)
	{
		super.createPartControl(parent);

		getEditorSite().getActionBars().setGlobalActionHandler(
			IWorkbenchActionConstants.UNDO,
			getActionRegistry().getAction(GEFActionConstants.UNDO));

		getEditorSite().getActionBars().setGlobalActionHandler(
			IWorkbenchActionConstants.REDO,
			getActionRegistry().getAction(GEFActionConstants.REDO));
			
		getEditorSite().getActionBars().setGlobalActionHandler(
			IWorkbenchActionConstants.DELETE,
			getActionRegistry().getAction(GEFActionConstants.DELETE));
	}
	public Object getAdapter(Class aClass)
	{
		if (aClass == CommandStack.class)
			return getCommandStack();
		else if (aClass == IPropertySheetPage.class)
			return getPropertySheetPage();
		else if (aClass == IContentOutlinePage.class)			
            return createContentOutlinePage();
		return super.getAdapter(aClass);
	}
    
    public DiagramOutlinePage createContentOutlinePage()
    {
        if (getGraphicalViewer() != null && getGraphicalViewer().getContents() != null && getActionRegistry() != null)
        {
            DiagramOutlinePage page =
                new DiagramOutlinePage(getCommandStack(), getGraphicalViewer().getContents(), getActionRegistry());
            return page;
        }
        return null;
    }
	
	public PropertySheetPage getPropertySheetPage()
	{
		if (propPage == null)
		{
			propPage = new PropertySheetPage();
			propPage.setRootEntry(GEFPlugin.createUndoablePropertySheetEntry(getCommandStack()));
		}
		return propPage;
	}
	
	
	
	
	
	
	
	
	
	
	
	
	public void doSave(IProgressMonitor monitor)
	{
        
	}
    

	public void doSaveAs()
	{
        System.out.println("yoooo");
        
        URI file = URI.createFileURI(ResourcesPlugin.getWorkspace().getRoot().getProject(content.getProjectName()).getFolder(content.getModuleName()).getLocation().toString()+"/savedModelJinha.xmi");
        modelManager.getResource().setURI(file);
        
            

                modelManager.save(file);

       
   
	}
	public void gotoMarker(IMarker marker)
	{
	}
	public boolean isDirty()
	{
		return false;
	}
	public boolean isSaveAsAllowed()
	{
		return true;
	}

    /**
     * @return Returns the modelManager.
     */
    public static EMFModelManager getModelManager() {
        return modelManager;

    }
    /**
     * @param modelManager The modelManager to set.
     */
    public void setModelManager(EMFModelManager modemManager) {
        this.modelManager = modemManager;
    }
    /**
     * @return Returns the isInitialized.
     */
    public boolean isInitialized() {
        return isInitialized;
    }
    /**
     * @param isInitialized The isInitialized to set.
     */
    public void setInitialized(boolean isInitialized) {
        this.isInitialized = isInitialized;
    }
    /**
     * @return Returns the fileInput.
     */
    public IFile getFileInput() {
        return fileInput;
    }
    /**
     * @param fileInput The fileInput to set.
     */
    public void setFileInput(IFile fileInput) {
        this.fileInput = fileInput;
    }
}
