package org.objectweb.jinha.editor;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.gef.palette.CombinedTemplateCreationEntry;
import org.eclipse.gef.palette.ConnectionCreationToolEntry;
import org.eclipse.gef.palette.PaletteContainer;
import org.eclipse.gef.palette.PaletteDrawer;
import org.eclipse.gef.palette.PaletteEntry;
import org.eclipse.gef.palette.PaletteGroup;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.palette.PaletteSeparator;
import org.eclipse.gef.palette.SelectionToolEntry;
import org.eclipse.gef.palette.ToolEntry;
import org.eclipse.gef.requests.SimpleFactory;
import org.objectweb.jinha.model.AttributeModel;
import org.objectweb.jinha.model.BusinessMethodModel;
import org.objectweb.jinha.model.CMPModel;
import org.objectweb.jinha.model.EntityModel;
import org.objectweb.jinha.model.FinderModel;
import org.objectweb.jinha.model.HomeMethodModel;
import org.objectweb.jinha.model.MessageDrivenModel;
import org.objectweb.jinha.model.ModelCreationFactory;
import org.objectweb.jinha.model.RelationModel;
import org.objectweb.jinha.model.SessionModel;


/**
 * Handles the creation of the palette for the Flow Editor.
 * 
 * @author Daniel Lee
 */
public class GlobalPalette {

    Resource res=null;
    /**
     * Constructor
     */
    public GlobalPalette(Resource r) {
        res=r;
    }

    /**
     * @param root
     *            the root of the palette
     * @return the list of categories which are added to the root
     */
    private List createCategories(PaletteRoot root) {
        List categories = new ArrayList();
        paletteContainer = createControlGroup(root);
        categories.add(paletteContainer);
       categories.add(createEjbDrawer());
        categories.add(createMethodDrawer());
        categories.add(createFieldDrawer());
        return categories;
    }

    /**
     * @return a container for creating EJB
     */
    private  PaletteContainer createEjbDrawer() {
        PaletteDrawer drawer=null;
       
            drawer = new PaletteDrawer("EJB", null);
        List entries = new ArrayList();
        CombinedTemplateCreationEntry combined = new CombinedTemplateCreationEntry(
                "Session",
                "Create a new Session Bean",
                SessionModel.class,
                new ModelCreationFactory(SessionModel.class,res),
                /*ImageDescriptor.createFromFile(JINHa.class,"images/session.gif"),
                ImageDescriptor.createFromFile(JINHa.class,"images/session.gif"));
       */
                null,null);
        entries.add(combined);
        combined = new CombinedTemplateCreationEntry(
                "Entity",
                "Create a new Entity Bean",
                EntityModel.class,
                new ModelCreationFactory(EntityModel.class,res),
                /*ImageDescriptor.createFromFile(JINHa.class,"images/session.gif"),
                ImageDescriptor.createFromFile(JINHa.class,"images/session.gif"));
       */
                null,null);
        entries.add(combined);
        combined = new CombinedTemplateCreationEntry(
                "MDB",
                "Create a new MessageDriven Bean",
                MessageDrivenModel.class,
                new ModelCreationFactory(MessageDrivenModel.class,res),
                /*ImageDescriptor.createFromFile(JINHa.class,"images/session.gif"),
                ImageDescriptor.createFromFile(JINHa.class,"images/session.gif"));
       */
                null,null);
        entries.add(combined);
        drawer.addAll(entries);
        return drawer;
    }

    private  PaletteContainer createMethodDrawer() {
        PaletteDrawer drawer=null;;
       
            drawer = new PaletteDrawer("METHODS", null);
        List entries = new ArrayList();
        CombinedTemplateCreationEntry combined = new CombinedTemplateCreationEntry(
                "Business",
                "Create a new Business Method",
                SessionModel.class,
                new ModelCreationFactory(BusinessMethodModel.class,res),
                /*ImageDescriptor.createFromFile(JINHa.class,"images/session.gif"),
                ImageDescriptor.createFromFile(JINHa.class,"images/session.gif"));
       */
                null,null);
        entries.add(combined);
        combined = new CombinedTemplateCreationEntry(
                "Home",
                "Create a new Home Method",
                SessionModel.class,
                new ModelCreationFactory(HomeMethodModel.class,res),
                /*ImageDescriptor.createFromFile(JINHa.class,"images/session.gif"),
                ImageDescriptor.createFromFile(JINHa.class,"images/session.gif"));
       */
                null,null);
        entries.add(combined);
        combined = new CombinedTemplateCreationEntry(
                "Finder",
                "Create a new Finder",
                SessionModel.class,
                new ModelCreationFactory(FinderModel.class,res),
                /*ImageDescriptor.createFromFile(JINHa.class,"images/session.gif"),
                ImageDescriptor.createFromFile(JINHa.class,"images/session.gif"));
       */
                null,null);
        entries.add(combined);
        drawer.addAll(entries);
        return drawer;
    }
    
    private  PaletteContainer createFieldDrawer() {
        PaletteDrawer drawer=null;;
       
            drawer = new PaletteDrawer("FIELDS", null);
        List entries = new ArrayList();
        CombinedTemplateCreationEntry combined = new CombinedTemplateCreationEntry(
                "Attribute",
                "Create a new Attribute",
                SessionModel.class,
                new ModelCreationFactory(AttributeModel.class,res),
                /*ImageDescriptor.createFromFile(JINHa.class,"images/session.gif"),
                ImageDescriptor.createFromFile(JINHa.class,"images/session.gif"));
       */
                null,null);
        entries.add(combined);
        combined = new CombinedTemplateCreationEntry(
                "CMP",
                "Create a CMP field",
                SessionModel.class,
                new ModelCreationFactory(CMPModel.class,res),
                /*ImageDescriptor.createFromFile(JINHa.class,"images/session.gif"),
                ImageDescriptor.createFromFile(JINHa.class,"images/session.gif"));
       */
                null,null);
        entries.add(combined);
        
        drawer.addAll(entries);
        return drawer;
    }
    
    PaletteContainer paletteContainer = null;

    /**
     * @return
     */
    public PaletteContainer getPaletteContainer() {
        return paletteContainer;
    }

    /**
     * @param root
     *            the root of the palette
     * @return the container for selection tool, marquee tool and connection
     *         creation
     */
    private PaletteContainer createControlGroup(PaletteRoot root) {
        PaletteGroup controlGroup = new PaletteGroup("Control Group");
        List entries = new ArrayList();
        ToolEntry tool = new SelectionToolEntry();
        entries.add(tool);
        root.setDefaultEntry(tool);
        /*tool = new MarqueeToolEntry();
        entries.add(tool);
        */
        PaletteSeparator sep = new PaletteSeparator("org.objectweb.jinha.sep1");
        sep.setUserModificationPermission(PaletteEntry.PERMISSION_NO_MODIFICATION);
        entries.add(sep);
        tool = new ConnectionCreationToolEntry(
                "relationship",
                "Creating a relationship",
                new ModelCreationFactory(RelationModel.class,res),
                null,//ImageDescriptor.createFromFile(JINHa.class,"images/connection.gif"),
                null);//ImageDescriptor.createFromFile(JINHa.class,"images/connection.gif"));
       
        entries.add(tool);
        

        System.out.println("entries" + entries);
        controlGroup.addAll(entries);
        return controlGroup;
    }
    PaletteRoot root=null;
    /**
     * Creates the PaletteRoot and adds all Palette elements.
     * 
     * @return the root
     */
    public PaletteRoot createPalette() {
        PaletteRoot flowPalette = new PaletteRoot();
        flowPalette.addAll(createCategories(flowPalette));
        root=flowPalette;
        return flowPalette;
    }

  
    /**
     * @return Returns the res.
     */
    public Resource getRes() {
        return res;
    }
    /**
     * @param res The res to set.
     */
    public void setRes(Resource res) {
        this.res = res;
    }
}