package org.objectweb.jinha.editparts;

import java.beans.PropertyChangeEvent;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditPolicy;
import org.objectweb.jinha.editpolicies.ComponentEditPolicyForwardDelete;
import org.objectweb.jinha.figures.AttributeFigure;
import org.objectweb.jinha.model.AttributeModel;


/**
 * the editpart for an attribute
 * catch event from the model and eventually represent it to the figure
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */

public class AttributeEditPart extends ObjectEditPart {

    
    /**
     * @return the figure corresponding to an attribute
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     */
    public IFigure createFigure() {
        return new AttributeFigure();
    }

    
    /**
     * 
     * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
     */
    public void refreshVisuals() {
        AttributeFigure figure = (AttributeFigure) getFigure();
        AttributeModel model = (AttributeModel) getModel();
        figure.setSignature(model.getVisibility(), model.getType(), model
                .getIdentifier());
    }

   
    /**
     * @return the list of children for an attribute
     * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
     */
    public List getModelChildren() {
        return new ArrayList();
    }

   
    /**
     * 
     * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
     */
    public void createEditPolicies() {
        super.createEditPolicies();
        installEditPolicy(EditPolicy.COMPONENT_ROLE,
                new ComponentEditPolicyForwardDelete() {
                });
    }

   
    /**
     * @param ev event caught and treated by the attribute editpart 
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent ev) {
        if (ev.getPropertyName().equals(
                AttributeModel.PROPERTY_SIGNATURE_CHANGED))
            refreshVisuals();
        else
            super.propertyChange(ev);
    }
}