package org.objectweb.jinha.editparts;
 
import java.beans.PropertyChangeEvent;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditPolicy;
import org.objectweb.jinha.editpolicies.ComponentEditPolicyForwardDelete;
import org.objectweb.jinha.figures.MethodFigure;
import org.objectweb.jinha.model.AttributeModel;
import org.objectweb.jinha.model.BusinessMethodModel;

/**
 *  the editpart for an BusinessMethod 
 * catch event from the model and eventually represent it to the figure
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */

public class BusinessMethodEditPart extends ObjectEditPart {

  
    /**
     * @return the figure that represent a business method
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     */
    public IFigure createFigure() {
        return new MethodFigure();
    }

   
    /**
     * refresh the visibility and the signature of th method
     * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
     */
    public void refreshVisuals() {
        MethodFigure figure = (MethodFigure) getFigure();
        BusinessMethodModel model = (BusinessMethodModel) getModel();
        figure.setSignature(model.getVisibility(), model.getReturnType(), model
                .getIdentifier(), model.getParametersTypes());
    }

   
    /**
     * @return the lsit of children for a business method
     * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
     */
    public List getModelChildren() {
        return new ArrayList();
    }

    /**
     *  define policies allowed for this part
     * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
     */
    public void createEditPolicies() {
        super.createEditPolicies();
        installEditPolicy(EditPolicy.COMPONENT_ROLE,
                new ComponentEditPolicyForwardDelete() {
                });
    }


    /**
     * @param ev the event caght and treated by this part
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent ev) {
        if (ev.getPropertyName().equals(
                AttributeModel.PROPERTY_SIGNATURE_CHANGED))
            refreshVisuals();
        else
            super.propertyChange(ev);
    }
}