package org.objectweb.jinha.editparts;

import java.beans.PropertyChangeEvent;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditPolicy;
import org.objectweb.jinha.editpolicies.ComponentEditPolicyForwardDelete;
import org.objectweb.jinha.figures.CMPFigure;
import org.objectweb.jinha.model.AttributeModel;
import org.objectweb.jinha.model.CMPModel;

/**
 * the editpart for an CMP catch event from the model and eventually represent
 * it to the figure
 * 
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 */
public class CMPEditPart extends ObjectEditPart {

    /**
     * @return the figure associated to a CMP
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     */
    public IFigure createFigure() {
        return new CMPFigure();
    }

    /**
     * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
     */
    public void refreshVisuals() {
        CMPFigure figure = (CMPFigure) getFigure();
        CMPModel model = (CMPModel) getModel();
        figure.setSignature(model.getReturnType(), model.getIdentifier(), model
                .getParametersTypes(),model.isPrimaryKey());
    }

    /**
     * @return the lsit of children for this part
     * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
     */
    public List getModelChildren() {
        return new ArrayList();
    }

    /**
     * defines policies allowed for this part
     * 
     * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
     */
    public void createEditPolicies() {
        super.createEditPolicies();
        installEditPolicy(EditPolicy.COMPONENT_ROLE,
                new ComponentEditPolicyForwardDelete() {
                });
    }

    /**
     * @param ev
     *            event caught and treated by this part
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent ev) {
        if (ev.getPropertyName().equals(
                AttributeModel.PROPERTY_SIGNATURE_CHANGED))
            refreshVisuals();
        else
            super.propertyChange(ev);
    }
}




