package org.objectweb.jinha.editparts;
 
import java.beans.PropertyChangeEvent;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditPolicy;
import org.objectweb.jinha.editpolicies.ComponentEditPolicyForwardDelete;
import org.objectweb.jinha.figures.CMRFigure;
import org.objectweb.jinha.model.AttributeModel;
import org.objectweb.jinha.model.CMRModel;

/**
 *  the editpart for a CMR
 * catch event from the model and eventually represent it to the figure
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class CMREditPart extends ObjectEditPart {

    /** 
     * @return the figure associated for a CMR
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     */
    public IFigure createFigure() {
        return new CMRFigure();
    }

    /**
     * 
     * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
     */
    public void refreshVisuals() {
        CMRFigure figure = (CMRFigure) getFigure();
        CMRModel model = (CMRModel) getModel();
        figure.setSignature(model.getVisibility(), model.getReturnType(), model
                .getIdentifier(), model.getParametersTypes());
    }

    /**
     * @return ths list of children for this part
     * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
     */
    public List getModelChildren() {
        return new ArrayList();
    }

    /**
     * define policies allowed by this part
     * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
     */
    public void createEditPolicies() {
        super.createEditPolicies();
        installEditPolicy(EditPolicy.COMPONENT_ROLE,
                new ComponentEditPolicyForwardDelete() {
                });
    }

    /**
     * @param ev event caught and treated by this part
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent ev) {
        if (ev.getPropertyName().equals(
                AttributeModel.PROPERTY_SIGNATURE_CHANGED))
            refreshVisuals();
        else
            super.propertyChange(ev);
    }
}