package org.objectweb.jinha.editparts;
 
import java.beans.PropertyChangeEvent;
import java.util.ArrayList;
import java.util.List;
import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.objectweb.jinha.editpolicies.EjbComponentEditPolicy;
import org.objectweb.jinha.editpolicies.GraphicalNodeEditPolicyImpl;
import org.objectweb.jinha.figures.EntityFigure;
import org.objectweb.jinha.model.EntityModel;
import org.objectweb.jinha.model.SessionModel;
import org.objectweb.jinha.model.properties.PropertyEjbBeanEntity;

public class EntityEditPart extends DiagramElementEditPart implements NodeEditPart
{
	private ChopboxAnchor anchor;
	
	public IFigure createFigure()
	{
		IFigure fig = new EntityFigure();
		anchor = new ChopboxAnchor(fig);
		return fig;
	}
	public void refreshVisuals()
	{
		super.refreshVisuals();
		EntityFigure figure = (EntityFigure)getFigure();
		EntityModel model = (EntityModel)getModel();
		figure.setLabel(model.getPropertyEjbBeanEntity().getEjbBeanName());
	}
	public List getModelChildren()
	{
		List children = new ArrayList();
		EntityModel model = (EntityModel)getModel();
		children.add(model.getAttributeList());
		children.add(model.getBusinessMethodsList());
        children.add(model.getHomeMethods());
        children.add(model.getCMPs());
        children.add(model.getCMRs());
        children.add(model.getFinders());
		return children;
	}
	public List getModelSourceConnections()
	{
		return ((EntityModel)getModel()).getSourceConnections();
	}
	public List getModelTargetConnections()
	{
		return ((EntityModel)getModel()).getTargetConnections();
	}
	public IFigure getContentPane()
	{
		return ((EntityFigure)getFigure()).getContentPane();
	}
    
    
    
    public AttributeListEditPart getAttributeListEditPart()
    {
        return (AttributeListEditPart)getChildren().get(0);
    }
	public BusinessMethodListEditPart getBusinessMethodListEditPart()
	{
		return (BusinessMethodListEditPart)getChildren().get(1);
	}
    public HomeMethodListEditPart getHomeMethodListEditPart()
    {
        return (HomeMethodListEditPart)getChildren().get(2);
    }  
    public CMPListEditPart getCMPListEditPart()
    {
        return (CMPListEditPart)getChildren().get(3);
    }
    public CMRListEditPart getCMRListEditPart()
    {
        return (CMRListEditPart)getChildren().get(4);
    }
    public FinderListEditPart getFinderListEditPart()
    {
        return (FinderListEditPart)getChildren().get(5);
    }
    
    
    
    
	public void propertyChange(PropertyChangeEvent ev)
	{
		if(ev.getPropertyName().equals(PropertyEjbBeanEntity.PROPERTY_NAME_CHANGED))
			refreshVisuals();
		else if(ev.getPropertyName().equals(EntityModel.PROPERTY_SOURCE_CONNECTIONS_CHANGED))
			refreshSourceConnections();
		else if(ev.getPropertyName().equals(EntityModel.PROPERTY_TARGET_CONNECTIONS_CHANGED))
			refreshTargetConnections();
			super.propertyChange(ev);
	}
	public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection)
	{
		return anchor;
	}
	public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection)
	{
		return anchor;
	}
	public ConnectionAnchor getSourceConnectionAnchor(Request request)
	{
		return anchor;
	}
	public ConnectionAnchor getTargetConnectionAnchor(Request request)
	{
		return anchor;
	}
	public void createEditPolicies()
	{
		super.createEditPolicies();
		installEditPolicy(EditPolicy.NODE_ROLE, new GraphicalNodeEditPolicyImpl());
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new EjbComponentEditPolicy(){});
	}
    
     /**
     * 
     * @see org.eclipse.gef.EditPart#activate()
     */
    public void activate() {
        // TODO Auto-generated method stub
        
        if (!isActive()) {
            super.activate();
            (((EntityModel) getModel()).getPropertyEjbBeanEntity()).addPropertyChangeListener(this);
        }
    }
    /**
     * 
     * @see org.eclipse.gef.EditPart#deactivate()
     */
    public void deactivate() {
        // TODO Auto-generated method stub
        super.deactivate();
        (((EntityModel) getModel()).getPropertyEjbBeanEntity()).removePropertyChangeListener(this);
    }
}
