package org.objectweb.jinha.editparts;
 
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editparts.AbstractConnectionEditPart;
import org.eclipse.gef.editpolicies.ConnectionEndpointEditPolicy;
import org.objectweb.jinha.editpolicies.BendpointEditPolicyImpl;
import org.objectweb.jinha.editpolicies.RelationEditPolicyImpl;
import org.objectweb.jinha.figures.RelationFigure;
import org.objectweb.jinha.model.CMRModel;
import org.objectweb.jinha.model.ObjectModel;
import org.objectweb.jinha.model.RelationModel;


/**
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class RelationEditPart extends AbstractConnectionEditPart implements
        PropertyChangeListener {

    //private EditPart source;
    //private EditPart target;
    RelationFigure  figure      = new RelationFigure();

    // this boolean permits to determine the good time to add a listener
    // on the target EJB (cause at the creation ,unidirectionnal relation means
    // CMRtarget) is null
    private boolean isCmrListen = false;

    /**
     * @return the figure associated with the Relation edit part
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     */
    protected IFigure createFigure() {
        return figure;

    }

    
    /**
     * 
     * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
     */
    protected void refreshVisuals() {
       // it remove the previous multiplicity
        try {
            figure.remove(figure.getSrcMany());
 
        }
        catch (Exception e) {
      
        } 
        try {
            
            figure.remove(figure.getSrcOne());

        }
        catch (Exception e) {
  
        }
        try {
       
            figure.remove(figure.getTargetMany());

        }
        catch (Exception e) {
   
        }
        try {
 
            figure.remove(figure.getTargetOne());
        }
        catch (Exception e) {
 
        }

        figure.setTargetDecoration(null);
        figure.setSourceDecoration(null);

        // get the CMRs source and target
        Object constraint = ((RelationModel) getModel()).getConstraint();
        CMRModel cmrSrc = ((RelationModel) getModel()).getCMRsrc();
        CMRModel cmrTarget = ((RelationModel) getModel()).getCMRtarget();

        // if the CMR source exist, it choose the correct decoration
        if (cmrSrc != null) {
            figure.setTargetDecoration(figure.getTarget());
//            String srcMultiplicity = CMRModel.multiplicityType[cmrSrc
//                    .getMultiplicity()];
            String srcMultiplicity = cmrSrc.getMultiplicity();
            if (srcMultiplicity.equals("one")) {
                              
                figure.add(figure.getTargetOne(), figure.getTargetEndpointLocator());
                
            }
            else {
               
                figure.add(figure.getTargetMany(), figure
                        .getTargetEndpointLocator());
                
            }
        }

        // Same thing for the target
        if (cmrTarget != null) {
            figure.setSourceDecoration(figure.getSrc());
//            String targetMultiplicity = CMRModel.multiplicityType[cmrTarget
//                    .getMultiplicity()];
            String targetMultiplicity = cmrTarget.getMultiplicity();
            if (targetMultiplicity.equals("one")) {
                
               
                figure.add(figure.getSrcOne(), figure
                        .getSourceEndpointLocator());
            }
            
            else {
               
                figure.add(figure.getSrcMany(), figure
                        .getSourceEndpointLocator());
            }
        }
        figure.setRoutingConstraint(constraint);

    }

    
    /**
     * @param ev
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent ev) {
        // get the CMR Target
        RelationModel relationModel = (RelationModel) getModel();
        CMRModel cmrModelTarget = relationModel.getCMRtarget();

        // In case of a deleted CMR Field that is listened by the edit Part
        if (ev.getPropertyName().equals(CMRModel.PROPERTY_MULTIPLICITY_CHANGED)) {

            if (cmrModelTarget == null && isCmrListen == true)
                isCmrListen = false;
        }

        // Add listeners on the CMRs target fields
        if (cmrModelTarget != null && isCmrListen == false) {
            cmrModelTarget.addPropertyChangeListener(this);
            isCmrListen = true;
        }

        refreshVisuals();
    }

    
    /**
     * 
     * @see org.eclipse.gef.EditPart#activate()
     */
    public void activate() {
        if (!isActive()) {
            super.activate();
            ((ObjectModel) getModel()).addPropertyChangeListener(this);
            RelationModel relationModel = (RelationModel) getModel();

            // Add listeners on the CMRs Source fields
            CMRModel cmrModelSrc = relationModel.getCMRsrc();
            cmrModelSrc.addPropertyChangeListener(this);

        }
    }

    
    /**
     * 
     * @see org.eclipse.gef.EditPart#deactivate()
     */
    public void deactivate() {
        super.deactivate();
        ((ObjectModel) getModel()).removePropertyChangeListener(this);
        RelationModel relationModel = (RelationModel) getModel();

        // Remove listeners on the CMRs Source fields
        CMRModel cmrModelSrc = relationModel.getCMRsrc();
        if (cmrModelSrc != null)
            cmrModelSrc.removePropertyChangeListener(this);

        // Remove listeners on the CMRs target fields
        CMRModel cmrModelTarget = relationModel.getCMRsrc();
        if (cmrModelTarget != null)
            cmrModelTarget.removePropertyChangeListener(this);
    }

    
    /**
     * 
     * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
     */
    public void createEditPolicies() {
        installEditPolicy(EditPolicy.CONNECTION_ENDPOINTS_ROLE,
                new ConnectionEndpointEditPolicy());
        installEditPolicy(EditPolicy.CONNECTION_BENDPOINTS_ROLE,
                new BendpointEditPolicyImpl());
        installEditPolicy(EditPolicy.CONNECTION_ROLE,
                new RelationEditPolicyImpl());
    }

}