package org.objectweb.jinha.editpolicies;
 
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.BendpointEditPolicy;
import org.eclipse.gef.requests.BendpointRequest;
import org.objectweb.jinha.commands.CreateBendpointCommand;
import org.objectweb.jinha.commands.DeleteBendpointCommand;
import org.objectweb.jinha.commands.MoveBendpointCommand;
import org.objectweb.jinha.model.RelationModel;


/**
 * this policy catches requests designed for bendpoints and creates commands in output 
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class BendpointEditPolicyImpl extends BendpointEditPolicy
{

	/**
	 * @param request the request for creating a bendpoint
	 * @return the command which creates a bendpoint
	 * @see org.eclipse.gef.editpolicies.BendpointEditPolicy#getCreateBendpointCommand(org.eclipse.gef.requests.BendpointRequest)
	 */
	protected Command getCreateBendpointCommand(BendpointRequest request)
	{
		int index = request.getIndex();
		Point loc = request.getLocation();
		RelationModel model = (RelationModel)getHost().getModel();
		return new CreateBendpointCommand(model, index, loc);
	}

	/**
	 * @param request the request for deleting a bendpoint
	 * @return the command which deleted a bendpoint
	 * @see org.eclipse.gef.editpolicies.BendpointEditPolicy#getDeleteBendpointCommand(org.eclipse.gef.requests.BendpointRequest)
	 */
	protected Command getDeleteBendpointCommand(BendpointRequest request)
	{
		int index = request.getIndex();
		RelationModel model = (RelationModel)getHost().getModel();
		return new DeleteBendpointCommand(model, index);
	}

	/**
	 * @param request the request for moving a bendpoint
	 * @return the command which moving a bendpoint
	 * @see org.eclipse.gef.editpolicies.BendpointEditPolicy#getMoveBendpointCommand(org.eclipse.gef.requests.BendpointRequest)
	 */
	protected Command getMoveBendpointCommand(BendpointRequest request)
	{
		int index = request.getIndex();
		Point loc = request.getLocation();
		RelationModel model = (RelationModel)getHost().getModel();
		return new MoveBendpointCommand(model, index, loc);
	}

}
