package org.objectweb.jinha.editpolicies;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;
import org.objectweb.jinha.commands.CreateRelationCommand;
import org.objectweb.jinha.commands.ReconnectSourceCommand;
import org.objectweb.jinha.commands.ReconnectTargetCommand;
import org.objectweb.jinha.editparts.RelationEditPart;
import org.objectweb.jinha.model.EntityModel;
import org.objectweb.jinha.model.RelationModel;

/**
* this policy catches requests designed for node elements and creates commands for managing relations
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class GraphicalNodeEditPolicyImpl extends GraphicalNodeEditPolicy
{
   


	/**
	 * @param request the request that put the ejb source et create the relation
	 * @return the command that create the relation
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCompleteCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	protected Command getConnectionCompleteCommand(CreateConnectionRequest request)
	{
        EntityModel target = (EntityModel)getHost().getModel();
        CreateRelationCommand command = (CreateRelationCommand) request.getStartCommand();		
        command.setTarget(target);     		
        return command;

	}

	/**
	 * @param request the request that give the source ejb
	 * @return the command that create the relation
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	protected Command getConnectionCreateCommand(CreateConnectionRequest request)
	{
        
        EntityModel source= source = (EntityModel)getHost().getModel();
		CreateRelationCommand command = new CreateRelationCommand(source,null); 
        request.setStartCommand(command);
		return command;

	}

	/**
	 * @param request the request that let's reconnect a relation to a target ejb
	 * @return the ReconnectTargetCommand 
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectTargetCommand(org.eclipse.gef.requests.ReconnectRequest)
	 */
	protected Command getReconnectTargetCommand(ReconnectRequest request)
	{
		RelationEditPart part = (RelationEditPart)request.getConnectionEditPart();
		RelationModel model = (RelationModel)part.getModel();
		EntityModel target = (EntityModel)getHost().getModel();
		return new ReconnectTargetCommand(target, model);
	}

	/**
	 * @param request the request that let's reconnect a relation to a target ejb
	 * @return ReconnectSourceCommand
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectSourceCommand(org.eclipse.gef.requests.ReconnectRequest)
	 */
	protected Command getReconnectSourceCommand(ReconnectRequest request)
	{
		RelationEditPart part = (RelationEditPart)request.getConnectionEditPart();
		RelationModel model = (RelationModel)part.getModel();
		EntityModel source = (EntityModel)getHost().getModel();
		return new ReconnectSourceCommand(source, model);
	}

}
