package org.objectweb.jinha.generate;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.IType;
import org.objectweb.jinha.editor.Editor;
import org.objectweb.jinha.model.emf.jinha.GraphicalEntity;
import org.objectweb.jinha.model.emf.jinha.GraphicalMessageDriven;
import org.objectweb.jinha.model.emf.jinha.GraphicalSession;
import org.objectweb.jinha.model.emf.jinha.JinhaFactory;
import org.objectweb.jinha.model.emf.jinha.JinhaPackage;
import org.objectweb.jinha.model.emf.jinha.ModuleEjbDefinition;

import com.objectlearn.j2ee.gmf.classgen.EjbBuilder;
import com.objectlearn.j2ee.gmf.emitter.EjbEmitter;
import com.objectlearn.j2ee.gmf.emitter.EntityEjbEmitter;
import com.objectlearn.j2ee.gmf.emitter.MessageDrivenEjbEmitter;
import com.objectlearn.j2ee.gmf.emitter.SessionEjbEmitter;
import com.objectlearn.lomboz.xml.lomboz.DocumentRoot;
import com.objectlearn.lomboz.xml.lomboz.EjbDefinition;
import com.objectlearn.lomboz.xml.lomboz.EntityBean;
import com.objectlearn.lomboz.xml.lomboz.LombozFactory;
import com.objectlearn.lomboz.xml.lomboz.LombozPackage;
import com.objectlearn.lomboz.xml.lomboz.MessageDrivenBean;
import com.objectlearn.lomboz.xml.lomboz.SessionBean;
import com.sun.java.xml.ns.j2ee.J2eeFactory;
import com.sun.java.xml.ns.j2ee.J2eePackage;


/**
 * This class makes generation of Lomboz Files (files with 
 * xdoclet tags)
 * It takes a ModuleEjbDefinition (EMF model) in parameter then generates
 * using methods of the package com.objectlearn.j2ee.gmf.emitter of Lomboz 
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class Generate {
    
    protected JinhaPackage jinhaPackage = JinhaPackage.eINSTANCE;

    protected JinhaFactory jinhaFactory = jinhaPackage.getJinhaFactory();

    protected J2eePackage j2eePackage = J2eePackage.eINSTANCE;

    protected J2eeFactory j2eeFactory = j2eePackage.getJ2eeFactory();

    protected LombozPackage lombozPackage = LombozPackage.eINSTANCE;

    protected LombozFactory lombozFactory = lombozPackage.getLombozFactory();
    
    protected ProjectJINHa project = new ProjectJINHa();
    
    IPackageFragmentRoot sourceFolder = null;
    
            
    // Cration de l'ejbBuilder
    /**
     * Create the final file with xdoclet tag for a bean
     * @param rootObject : The DocumentRoot which contains the EJB 
     * @param ejbEmitter : Object(from Lomboz) which realise the generation
     * @param comment : comment of the ejb from the javajet file
     * @param stub : stub of the ejb from the javajet file
     * @param method : method of the ejb from the javajet file
     * @param fields : fields of the ejb from the javajet file
     * @param className : className of the ejb from the javajet file
     * @param packageName : packageName of the ejb from the javajet file
     * @return IType 
     * @throws CoreException
     * @throws InterruptedException
     */
    public IType createType(DocumentRoot rootObject, EjbEmitter ejbEmitter, String comment, 
            String stub, String method, String fields, String className, String packageName,
            String moduleName,String projectName, Editor editor) throws CoreException, InterruptedException {
        IConfigurationElement[] configurationElements = Platform.getExtensionRegistry().getConfigurationElementsFor("com.objectlearn.j2ee.gmf.emitter.template");

        EjbBuilder ejbBuilder = new EjbBuilder();
        ejbBuilder.setConfigurationElement(configurationElements[0]);
        ejbBuilder.setMonitor(ejbEmitter.getMonitor());
        
        // TODO rcuprer le nom du projet en cours
        if(sourceFolder==null) 
            sourceFolder = project.createSourceFolder(editor, moduleName);
        
        ejbBuilder.setPackageFragmentRoot(sourceFolder);
        ejbBuilder.setDocRoot(rootObject);
        ejbBuilder.setTypeName(className);
        ejbBuilder.setPackageName(packageName);
        ejbBuilder.setTypeComment(comment);
        ejbBuilder.setTypeStub(stub);
        ejbBuilder.setMethodStub(method);
        ejbBuilder.setFields(fields);
        
        ejbBuilder.createType();
              
        return ejbBuilder.getCreatedType();
    }
    
    
    /**
     * String chaine is evaluated and returned whithout space
     * @param chaine 
     * @return String 
     */
    String Test_espace(String chaine){
        for(int i=0;i<chaine.length();i++)
            if(chaine.charAt(i)!='\n'&&chaine.charAt(i)!='\r')
                return chaine;
        return "";
    }
    
    /**
     * Creation of the EjbEmitter from a list of DocumentRoot
     * @param rootObject : list of the DocumentRoot
     * @param ejbEmitter : object(from Lomboz) which realise the generation
     * @param configurationElements : configuration of the current plateform
     * @param i
     */
    public EjbEmitter ejbEmitterCreate(List rootObject,EjbEmitter ejbEmitter,IConfigurationElement[] configurationElements,int i){
        try {
            if(((DocumentRoot)rootObject.get(i)).getEjb().getEntity()!=null)
                ejbEmitter = new EntityEjbEmitter(configurationElements[0]);
            if(((DocumentRoot)rootObject.get(i)).getEjb().getSession()!=null)
                ejbEmitter = new SessionEjbEmitter(configurationElements[0]);
            if(((DocumentRoot)rootObject.get(i)).getEjb().getMessageDriven()!=null)
                ejbEmitter = new MessageDrivenEjbEmitter(configurationElements[0]);
         } catch (ClassNotFoundException e) {
             // TODO Auto-generated catch block
             e.printStackTrace();
         } catch (InstantiationException e) {
             // TODO Auto-generated catch block
             e.printStackTrace();
         } catch (IllegalAccessException e) {
             // TODO Auto-generated catch block
             e.printStackTrace();
         } catch (CoreException e) {
             // TODO Auto-generated catch block
             e.printStackTrace();
         }
         return ejbEmitter;
    }
    
    /**
     * @param rootObject : The DocumentRoot which contains the EJB 
     * @param ejbEmitter : Object(from Lomboz) which realise the generation
     * @return String : comment of the ejb from the javajet file  
     */
    public String generateTypeComment(DocumentRoot rootObject, EjbEmitter ejbEmitter ){
    
    String comment ="";
    
    try {
        comment = ejbEmitter.emitTypeComment(rootObject);
    } catch (ClassNotFoundException e1) {
        // TODO Auto-generated catch block
        e1.printStackTrace();
    } catch (InstantiationException e1) {
        // TODO Auto-generated catch block
       e1.printStackTrace();
    } catch (IllegalAccessException e1) {
        // TODO Auto-generated catch block
       e1.printStackTrace();
    } catch (CoreException e1) {
        // TODO Auto-generated catch block
       e1.printStackTrace();
    }
    return comment;
  }
    
    /**
     * @param rootObject : The DocumentRoot which contains the EJB 
     * @param ejbEmitter : Object(from Lomboz) which realise the generation
     * @return String : stub of the ejb from the javajet file
     */
    public String generateTypeStub(DocumentRoot rootObject, EjbEmitter ejbEmitter ){
        
        String stub ="";
        
        try {
            stub = ejbEmitter.emitTypeStub(rootObject);
        } catch (ClassNotFoundException e2) {
            // TODO Auto-generated catch block
            e2.printStackTrace();
        } catch (InstantiationException e2) {
            // TODO Auto-generated catch block
            e2.printStackTrace();
        } catch (IllegalAccessException e2) {
            // TODO Auto-generated catch block
            e2.printStackTrace();
        } catch (CoreException e2) {
            // TODO Auto-generated catch block
            e2.printStackTrace();
        }
        return stub;
      }
    
    /**
     * @param rootObject : The DocumentRoot which contains the EJB 
     * @param ejbEmitter : Object(from Lomboz) which realise the generation
     * @return String : Fields of the ejb from the javajet file
     */
    public String generateTypeField(DocumentRoot rootObject, EjbEmitter ejbEmitter ){
        
        String field ="";
        
        try {
            field = ejbEmitter.emitFields(rootObject);
        } catch (ClassNotFoundException e3) {
            // TODO Auto-generated catch block
            e3.printStackTrace();
        } catch (InstantiationException e3) {
            // TODO Auto-generated catch block
            e3.printStackTrace();
        } catch (IllegalAccessException e3) {
            // TODO Auto-generated catch block
            e3.printStackTrace();
        } catch (CoreException e3) {
            // TODO Auto-generated catch block
            e3.printStackTrace();
        }
        return field;
      }
    
    /**
     * @param rootObject : The DocumentRoot which contains the EJB 
     * @param ejbEmitter : Object(from Lomboz) which realise the generation
     * @return String : methods of the ejb from the javajet file
     */   
    public String generateTypeMethod(DocumentRoot rootObject, EjbEmitter ejbEmitter ){
        
        String method ="";
        
       try {
            method = ejbEmitter.emitInterfaceMethods(rootObject);
        } catch (ClassNotFoundException e3) {
            // TODO Auto-generated catch block
            e3.printStackTrace();
        } catch (InstantiationException e3) {
            // TODO Auto-generated catch block
            e3.printStackTrace();
        } catch (IllegalAccessException e3) {
            // TODO Auto-generated catch block
            e3.printStackTrace();
        } catch (CoreException e3) {
            // TODO Auto-generated catch block
            e3.printStackTrace();
        }
        return method;
    }
    
    
        
    /**
     * This function take a ModuleEjbDefinition on parameter
     * then generate Lomboz files 
     * @param module
     */
    public void generateLombozFile(ModuleEjbDefinition module,String moduleName,String projectName,Editor editor){

        List rootObject = ModuleToListDocumentRoot(module);
        
        for(int i=0;i<rootObject.size();i++){
            
         // Configuration d'EjbEmitter
         IConfigurationElement[] configurationElements = Platform.getExtensionRegistry().getConfigurationElementsFor("com.objectlearn.j2ee.gmf.emitter.template");
         EjbEmitter ejbEmitter = null;

         ejbEmitter = ejbEmitterCreate(rootObject,ejbEmitter,configurationElements,i);
         
         String comment = "";
         comment = generateTypeComment((DocumentRoot)rootObject.get(i),ejbEmitter);  

         String stub = "";
         stub = generateTypeStub((DocumentRoot)rootObject.get(i),ejbEmitter);
         
         String field= "";
         field = generateTypeField((DocumentRoot)rootObject.get(i),ejbEmitter);
         
         String method = "";
         method = generateTypeMethod((DocumentRoot)rootObject.get(i),ejbEmitter);

         String className = "";
         String packageName = "";
         
         try {
            if(((DocumentRoot)rootObject.get(i)).getEjb().getEntity()!=null){
                className = ((DocumentRoot)rootObject.get(i)).getEjb().getEntity().getEntityEjb().getEjbClass().getUnqualifiedClassName();
                packageName =((DocumentRoot)rootObject.get(i)).getEjb().getEntity().getEntityEjb().getEjbClass().getPackageName();
             }
             if(((DocumentRoot)rootObject.get(i)).getEjb().getSession()!=null){
                className = ((DocumentRoot)rootObject.get(i)).getEjb().getSession().getSessionEjb().getEjbClass().getUnqualifiedClassName();
                packageName =((DocumentRoot)rootObject.get(i)).getEjb().getSession().getSessionEjb().getEjbClass().getPackageName();
             }
             if(((DocumentRoot)rootObject.get(i)).getEjb().getMessageDriven()!=null){
                className = ((DocumentRoot)rootObject.get(i)).getEjb().getMessageDriven().getMessageDrivenEjb().getEjbClass().getUnqualifiedClassName();
                packageName =((DocumentRoot)rootObject.get(i)).getEjb().getMessageDriven().getMessageDrivenEjb().getEjbClass().getPackageName();
             }
        } catch (RuntimeException e4) {
            // TODO Auto-generated catch block
            e4.printStackTrace();
        }
         IType theType = null;
         
         // renvoie une chaine vide si il n'y a que des espaces divers
         field = Test_espace(field);
         method = Test_espace(method);
         comment = Test_espace(comment);
         stub = Test_espace(stub);
        
         try {
             theType = createType((DocumentRoot)rootObject.get(i), ejbEmitter, comment, stub, method, 
                     field , className, packageName, moduleName, projectName, editor);
             //assertTrue( xmlDef.length() > 0);
         } catch (CoreException e5) {
             // TODO Auto-generated catch block
             e5.printStackTrace();
         } catch (InterruptedException e5) {
             // TODO Auto-generated catch block
             e5.printStackTrace();
         }
                       
     }
        //On ajoute les beans au fichier XML
        //project.AddBeanXML(moduleName,rootObject);
    }
    /**
     * @return a list of a DocumentRoot from a ModuleEjbDefinition 
     */
    public List ModuleToListDocumentRoot(ModuleEjbDefinition module){
        List rootObject = new ArrayList();
        int i=0;
        
        for(i=0;i<module.getGraphicalEntityBeans().size();i++){
            GraphicalEntity graphicalEntity = (GraphicalEntity) module.getGraphicalEntityBeans().get(i);
            EntityBean entityBean = graphicalEntity.getEntityBean();
            EjbDefinition ejbDefinition = lombozFactory.createEjbDefinition();
            ejbDefinition.setEntity(entityBean);
            DocumentRoot documentRoot = lombozFactory.createDocumentRoot();
            documentRoot.setEjb(ejbDefinition);
            rootObject.add(documentRoot);
        }
        for(i=0;i<module.getGraphicalSessionBeans().size();i++){
            GraphicalSession graphicalSession = (GraphicalSession) module.getGraphicalSessionBeans().get(i);
            SessionBean sessionBean = graphicalSession.getSessionBean();
            EjbDefinition ejbDefinition = lombozFactory.createEjbDefinition();
            ejbDefinition.setSession(sessionBean);
            DocumentRoot documentRoot = lombozFactory.createDocumentRoot();
            documentRoot.setEjb(ejbDefinition);
            rootObject.add(documentRoot);
        }
        for(i=0;i<module.getGraphicalMessageDrivenBeans().size();i++){
            GraphicalMessageDriven graphicalMessageDriven = (GraphicalMessageDriven) module.getGraphicalMessageDrivenBeans().get(i);
            MessageDrivenBean messageDrivenBean = graphicalMessageDriven.getMessageDrivenBean();
            EjbDefinition ejbDefinition = lombozFactory.createEjbDefinition();
            ejbDefinition.setMessageDriven(messageDrivenBean);
            DocumentRoot documentRoot = lombozFactory.createDocumentRoot();
            documentRoot.setEjb(ejbDefinition);
            rootObject.add(documentRoot);
        }
        return rootObject;
    }
}





 
