package org.objectweb.jinha.model;

import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import com.objectlearn.lomboz.xml.lomboz.Attribute;
import com.sun.java.xml.ns.j2ee.JavaIdentifierType;
import com.sun.java.xml.ns.j2ee.JavaTypeType;


/**
 * this class is the model of an attribute
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class AttributeModel extends ObjectModel
{
	private String identifier;
	private String type;
	private String visibility;
    //ref emf lomboz
    private Attribute attribute=null;
	
	private IPropertyDescriptor [] descriptors;

	public static final String PROPERTY_SIGNATURE_CHANGED = "signature";
	public static final String PUBLIC = "public";
	public static final String PRIVATE = "private";
	public static final String PROTECTED = "protected";

	public static final String ID_PROPERTY_IDENTIFIER = "identifier";
	public static final String ID_PROPERTY_TYPE = "return type";
	public static final String ID_PROPERTY_VISIBILITY = "visibility";

	/**
	 * the constructor
	 */
	public AttributeModel(String name, Resource res)
	{
		setSignature(PRIVATE, "int", name);
        factory.setTargetClass(Attribute.class);
        factory.setRes(res);
        attribute=(Attribute)factory.getNewObject();
	}
	/**
	 * @param visibility
	 * @param type
	 * @param identifier
	 */
	public AttributeModel(String visibility, String type, String identifier)
	{
		setSignature(visibility, type, identifier);
	}
	/**
	 * @param visibility
	 * @param type
	 * @param identifier
	 */
	public void setSignature(String visibility, String type, String identifier)
	{
		this.visibility = visibility;
		this.type = type;
		this.identifier = identifier;
		firePropertyChange(PROPERTY_SIGNATURE_CHANGED, null, null);
	}
	/**
	 * @return
	 */
	public String getVisibility()
	{
		return visibility;
	}
	/**
	 * @return
	 */
	public String getType()
	{
		return type;
	}
	/**
	 * @return
	 */
	public String getIdentifier()
	{
		return identifier;
	}
	/**
	 * @return
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		return type + " " + identifier;
	}
	/**
     * @param type
     */
    public void setType(String type)
    {

        this.type = type;
        
        // EMF Part Begin
        factory.setTargetClass(JavaTypeType.class);
        JavaTypeType javaType =(JavaTypeType)factory.getNewObject(); 
        javaType.setValue(getType());
        getAttribute().setType(javaType);
        // EMF Part End
        
        firePropertyChange(PROPERTY_SIGNATURE_CHANGED, null, null);
    }
    /**
     * @param identifier
     */
    public void setIdentifier(String identifier)
    {
        this.identifier = identifier;
        
          // EMF Part Begin
        factory.setTargetClass(JavaIdentifierType.class);
        JavaIdentifierType javaIdentifier =(JavaIdentifierType)factory.getNewObject(); 
        javaIdentifier.setValue(getIdentifier());
        getAttribute().setName(javaIdentifier);
        // EMF Part End
        
        firePropertyChange(PROPERTY_SIGNATURE_CHANGED, null, null);
    }
    /**
     * @param visibility
     */
    public void setVisibility(String visibility)
    {
        if (visibility.equals("public") || visibility.equals("private")|| visibility.equals("protected"))
        {
            this.visibility = visibility;
            
            // EMF Part Begin
           
            getAttribute().getVisibility().setValue(getVisibility());
            // EMF Part End
                        
            firePropertyChange(PROPERTY_SIGNATURE_CHANGED, null, null);
        }
    }
	/**
	 * @return
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	public IPropertyDescriptor[] getPropertyDescriptors()
	{
		if (descriptors == null)
		{
			IPropertyDescriptor[] superDescriptors = super.getPropertyDescriptors();
			descriptors = new IPropertyDescriptor[superDescriptors.length + 3];
			for (int i = 0; i < superDescriptors.length; i++)
				descriptors[i] = superDescriptors[i];
			descriptors[superDescriptors.length] =
				new TextPropertyDescriptor(ID_PROPERTY_TYPE, "type");
			descriptors[superDescriptors.length + 1] =
				new TextPropertyDescriptor(ID_PROPERTY_IDENTIFIER, "identifier");
			descriptors[superDescriptors.length + 2] =
				new ComboBoxPropertyDescriptor(
					ID_PROPERTY_VISIBILITY,
					"visibility",
					new String[] { "public", "protected", "private" });
		}
		return descriptors;
	}
	/**
	 * @param propName
	 * @param val
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
	 */
	public void setPropertyValue(Object propName, Object val)
	{
		if (propName instanceof String)
		{
			if (((String) propName).equals(ID_PROPERTY_TYPE))
				setType(val.toString());
			else if (((String) propName).equals(ID_PROPERTY_IDENTIFIER))
				setIdentifier(val.toString());
			else if (((String) propName).equals(ID_PROPERTY_VISIBILITY))
			{
				if (((Integer) val).intValue() == 0)
					setVisibility(PUBLIC);
				else if (((Integer) val).intValue() == 2)
					setVisibility(PRIVATE);
				else if (((Integer) val).intValue() == 1)
					setVisibility(PROTECTED);
			}
		}
		else
			super.setPropertyValue(propName, val);
	}
	/**
	 * @param propName
	 * @return
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(Object propName)
	{
		if (propName instanceof String)
		{
			if (((String) propName).equals(ID_PROPERTY_TYPE))
				return type;
			else if (((String) propName).equals(ID_PROPERTY_IDENTIFIER))
				return identifier;
			else if (((String) propName).equals(ID_PROPERTY_VISIBILITY))
			{
				if (visibility == PUBLIC)
					return new Integer(0);
				else if (visibility == PRIVATE)
					return new Integer(2);
				else if (visibility == PROTECTED)
					return new Integer(1);
				else
					return super.getPropertyValue(propName);
			}
			else
				return super.getPropertyValue(propName);
		}
		else
			return super.getPropertyValue(propName);
	}
    /**
     * @return Returns the attribute.
     */
    public Attribute getAttribute() {
        return attribute;
    }
    /**
     * @param attribute The attribute to set.
     */
    public void setAttribute(Attribute attribute) {
        this.attribute = attribute;
    }
}