package org.objectweb.jinha.model;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;
import org.objectweb.jinha.model.properties.PropertyCmpPersistence;

import com.objectlearn.lomboz.xml.lomboz.FieldMapping;

/**
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class CMPModel extends ObjectModel
{
	private String identifier;
	private String returnType;
	private List parametersTypes;
	private IPropertyDescriptor[] descriptors;
    //private MethodType method=null;
    private FieldMapping fieldMapping=null; 
    private boolean isPrimaryKey=false;

	public static final String PROPERTY_SIGNATURE_CHANGED = "signature";
	public static final String PUBLIC = "public";
    public static final String PRIVATE = "private";
    public static final String PROTECTED = "protected";

	public static final String ID_PROPERTY_IDENTIFIER = "identifier";
	public static final String ID_PROPERTY_RETURN_TYPE = "return type";
	public static final String ID_PROPERTY_PK = "primaryKey";
	
	private static final String ID_PROPERTY_PERSISTENCE = "persistence";
   
    private PropertyCmpPersistence persistence;

	/**
	 * 
	 */
	public CMPModel(Resource res)
	{
        factory.setTargetClass(FieldMapping.class);
        factory.setRes(res);
        fieldMapping=(FieldMapping)factory.getNewObject();
		setSignature( "String", "PK_FIELD", new ArrayList(), false);
		
		  persistence = new PropertyCmpPersistence(getFieldMapping());
		  
	}
	
	/**
	 * @param visibility
	 * @param returnType
	 * @param identifier
	 * @param parametersTypes
	 * @param isPrimaryKey
	 */
	public CMPModel(String visibility, String returnType, String identifier, List parametersTypes, boolean isPrimaryKey)
	{
		setSignature( returnType, identifier, parametersTypes,isPrimaryKey);
		  persistence = new PropertyCmpPersistence(getFieldMapping());
	}
	
	/**
	 * @param visibility
	 * @param returnType
	 * @param identifier
	 * @param parametersTypes
	 * @param isPrimaryKey
	 */
	public void setSignature(String returnType, String identifier, List parametersTypes, boolean isPrimaryKey)
	{
        this.isPrimaryKey=isPrimaryKey;

		this.returnType = returnType;
		this.identifier = identifier;
		this.parametersTypes = parametersTypes;
		firePropertyChange(PROPERTY_SIGNATURE_CHANGED, null, null);
	}
	/**
	 * @return
	 */
	public String getReturnType()
	{
		return returnType;
	}
	/**
	 * @return
	 */
	public String getIdentifier()
	{
		return identifier;
	}
	/**
	 * @return
	 */
	public List getParametersTypes()
	{
		return parametersTypes;
	}
	/**
	 * @return
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		String txt = returnType + " " + identifier + "(";
		Iterator it = parametersTypes.iterator();
		if (it.hasNext())
			txt += it.next().toString();
		while (it.hasNext())
		{
			txt += " ," + it.next().toString();
		}
		txt += ")";
		return txt;
	}
	/**
	 * @param returnType
	 */
	public void setReturnType(String returnType)
	{
		this.returnType = returnType;
        
        // TODO EMF Link , add Property for CMP Field =>getFieldMapping().g
		
		firePropertyChange(PROPERTY_SIGNATURE_CHANGED, null, null);
	}
	/**
	 * @param identifier
	 */
	public void setIdentifier(String identifier)
	{
		this.identifier = identifier;
		firePropertyChange(PROPERTY_SIGNATURE_CHANGED, null, null);
	}

	
	/**
	 * @return
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	public IPropertyDescriptor[] getPropertyDescriptors()
	{
		if (descriptors == null)
		{
		    descriptors = (new IPropertyDescriptor[] {
				new TextPropertyDescriptor(ID_PROPERTY_RETURN_TYPE, ID_PROPERTY_RETURN_TYPE),
				new TextPropertyDescriptor(ID_PROPERTY_IDENTIFIER, ID_PROPERTY_IDENTIFIER),
				new ComboBoxPropertyDescriptor(
					ID_PROPERTY_PK,
					"primary key",
					new String[] { "true", "false" }),
				new PropertyDescriptor(ID_PROPERTY_PERSISTENCE, ID_PROPERTY_PERSISTENCE)
		    });
		}
		return descriptors;
	}
	/**
	 * @param propName
	 * @param val
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
	 */
	public void setPropertyValue(Object propName, Object val)
	{
		if (propName instanceof String)
		{
			if (((String) propName).equals(ID_PROPERTY_RETURN_TYPE))
				setReturnType(val.toString());
			else if (((String) propName).equals(ID_PROPERTY_IDENTIFIER))
				setIdentifier(val.toString());
			else if (((String) propName).equals(ID_PROPERTY_PK))
			{
			    if(((Integer) val).intValue() == 0)
			        setPrimaryKey(true);
			    else 
			        setPrimaryKey(false);
			}
				
		}
		else
			super.setPropertyValue(propName, val);
	}
	/**
	 * @param propName
	 * @return
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(Object propName)
	{
		if (propName instanceof String)
		{
			if (((String) propName).equals(ID_PROPERTY_RETURN_TYPE))
				return returnType;
			else if (((String) propName).equals(ID_PROPERTY_PERSISTENCE))
                return getPersistence();
            
			else if (((String) propName).equals(ID_PROPERTY_IDENTIFIER))
				return identifier;
			else if (((String) propName).equals(ID_PROPERTY_PK))
    			{
			    if(isPrimaryKey())
			        return new Integer(0);
			    else 
			        return new Integer(1);
    			}
			else 
			    return super.getPropertyValue(propName);
		}
		else
			return super.getPropertyValue(propName);
	}
    /**
     * @return Returns the fieldMapping.
     */
    public FieldMapping getFieldMapping() {
        return fieldMapping;
    }
    /**
     * @param fieldMapping The fieldMapping to set.
     */
    public void setFieldMapping(FieldMapping fieldMapping) {
        this.fieldMapping = fieldMapping;
    }
    /**
     * @return Returns the isPrimaryKey.
     */
    public boolean isPrimaryKey() {
        return isPrimaryKey;
    }
    /**
     * @param isPrimaryKey The isPrimaryKey to set.
     */
    public void setPrimaryKey(boolean isPrimaryKey) {
        this.isPrimaryKey = isPrimaryKey;
        fieldMapping.setPrimaryKey(isPrimaryKey);
        firePropertyChange(PROPERTY_SIGNATURE_CHANGED, null, null);
    }
    
    /**
     * @return Returns the persistence.
     */
    public PropertyCmpPersistence getPersistence() {
        return persistence;
    }
}