package org.objectweb.jinha.model;
 
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.objectweb.jinha.misc.IntegerPropertyDescriptor;
import org.objectweb.jinha.model.emf.jinha.GraphicalEJB;
import org.objectweb.jinha.model.emf.jinha.JinhaFactory;
import org.objectweb.jinha.model.emf.jinha.JinhaPackage;
import com.objectlearn.lomboz.xml.lomboz.LombozFactory;
import com.objectlearn.lomboz.xml.lomboz.LombozPackage;
import com.sun.java.xml.ns.j2ee.J2eeFactory;
import com.sun.java.xml.ns.j2ee.J2eePackage;

/*
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.objectweb.jinha.editor.misc.IntegerPropertyDescriptor;
*/

/**
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class DiagramElementModel extends ObjectModel
{
	private Point location;
	private Dimension size;

    private String  packageName;
    // Factory pour creer des references vers ejb lomboz

    
    LombozPackage lombozPackage = LombozPackage.eINSTANCE;
    LombozFactory lombozFactory = lombozPackage.getLombozFactory();
    J2eePackage j2eePackage = J2eePackage.eINSTANCE;
    J2eeFactory j2eeFactory = j2eePackage.getJ2eeFactory();
    
    JinhaPackage jinhaPackage = JinhaPackage.eINSTANCE;
    JinhaFactory jinhaFactory = jinhaPackage.getJinhaFactory();
    
    protected GraphicalEJB graphicalEjb;
    
	private static IPropertyDescriptor[] descriptors;

	public static final String PROPERTY_CONSTRAINT_CHANGED = "constraint";


    public static final String ID_PROPERTY_PACKAGE = "Package Name";
	public static final String ID_PROPERTY_LOCATION_X = "location x";
	public static final String ID_PROPERTY_LOCATION_Y = "location y";
	public static final String ID_PROPERTY_HEIGHT = "height";
	public static final String ID_PROPERTY_WIDTH = "width";

	/**
	 * @param location
	 */
	public void setLocation(Point location)
	{
		Point save = getLocation();
		this.location = location;
		graphicalEjb.setX(getLocation().x);
        graphicalEjb.setY(getLocation().y);
		firePropertyChange(PROPERTY_CONSTRAINT_CHANGED, save, location);
	}
	/**
	 * @return
	 */
	public Point getLocation()
	{
		return location;
	}
	/**
	 * @param size
	 */
	public void setSize(Dimension size)
	{
		Dimension save = getSize();
		this.size = size;
		graphicalEjb.setWidth(getSize().width);
        graphicalEjb.setHeight(getSize().height);
		firePropertyChange(PROPERTY_CONSTRAINT_CHANGED, save, size);
	}
	/**
	 * @return
	 */
	public Dimension getSize()
	{
		return size;
	}
	
	/**
	 * @return
	 */
	public GraphicalEJB getGraphicalEjb()
    {
        return graphicalEjb;
    }

    /**
     * @param graphicalEjb
     */
    public void setGraphicalEjb(GraphicalEJB graphicalEjb)
    {
        this.graphicalEjb = graphicalEjb;
    }
	
	/**
	 * @param rect
	 */
	public void setConstraint(Rectangle rect)
	{
		Rectangle save = new Rectangle(getLocation(), getSize());
		this.size = rect.getSize();
		this.location = rect.getLocation();
		firePropertyChange(PROPERTY_CONSTRAINT_CHANGED, save, rect);
	}
	/**
	 * @return
	 */
	public Rectangle getConstraint()
	{
		return new Rectangle(getLocation(), getSize());
	}
	/**
	 * @return
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	public IPropertyDescriptor[] getPropertyDescriptors()
	{
		if (descriptors == null)
		{
			IPropertyDescriptor[] superDescriptors = super.getPropertyDescriptors();
			descriptors = new IPropertyDescriptor[superDescriptors.length ];
			for (int i = 0; i < superDescriptors.length; i++)
				descriptors[i] = superDescriptors[i];
			/*descriptors[superDescriptors.length] = new IntegerPropertyDescriptor(ID_PROPERTY_LOCATION_X, "x");
			descriptors[superDescriptors.length + 1] = new IntegerPropertyDescriptor(ID_PROPERTY_LOCATION_Y, "y");
			descriptors[superDescriptors.length + 2] = new IntegerPropertyDescriptor(ID_PROPERTY_HEIGHT, "height");
			descriptors[superDescriptors.length + 3] = new IntegerPropertyDescriptor(ID_PROPERTY_WIDTH, "width");*/
		}
		return descriptors;
	}
	/**
	 * @param propName
	 * @param val
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
	 */
	public void setPropertyValue(Object propName, Object val)
	{
		/*if (propName instanceof String)
		{
			if (((String) propName).equals(ID_PROPERTY_LOCATION_X))
				setLocation(new Point(Integer.parseInt(val.toString()), getLocation().y));
			else if (((String) propName).equals(ID_PROPERTY_LOCATION_Y))
				setLocation(new Point(getLocation().x, Integer.parseInt(val.toString())));
			else if (((String) propName).equals(ID_PROPERTY_WIDTH))
				setSize(new Dimension(Integer.parseInt(val.toString()), getSize().height));
			else if (((String) propName).equals(ID_PROPERTY_HEIGHT))
				setSize(new Dimension(getSize().width, Integer.parseInt(val.toString())));
		}
		else*/
			super.setPropertyValue(propName, val);
	}
	/**
	 * @param propName
	 * @return
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(Object propName)
	{
		/*if (propName instanceof String)
		{
			if (((String) propName).equals(ID_PROPERTY_LOCATION_X))
				return new Integer(getLocation().x).toString();
			else if (((String) propName).equals(ID_PROPERTY_LOCATION_Y))
				return new Integer(getLocation().y).toString();
			else if (((String) propName).equals(ID_PROPERTY_WIDTH))
				return new Integer(getSize().width).toString();
			else if (((String) propName).equals(ID_PROPERTY_HEIGHT))
				return new Integer(getSize().height).toString();
		}*/
		return super.getPropertyValue(propName);
	}


    /**
     * @return Returns the packageName.
     */
    public String getPackageName() {
        return packageName;
    }
    /**
     * @param packageName The packageName to set.
     */
    public void setPackageName(String packageName) {
        this.packageName = packageName;
    }
}