package org.objectweb.jinha.model;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import com.sun.java.xml.ns.j2ee.JavaTypeType;
import com.sun.java.xml.ns.j2ee.MethodType;



/**
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class HomeMethodModel extends ObjectModel
{
	private String identifier;
	private String returnType;
	private String visibility;
	private List parametersTypes;
	private IPropertyDescriptor[] descriptors;
    private MethodType method=null;
    private String parameters;

    private String viewType;
    
	public static final String PROPERTY_SIGNATURE_CHANGED = "signature";
	public static final String PUBLIC = "public";
	public static final String PRIVATE = "private";
	public static final String PROTECTED = "protected";

	public static final String ID_PROPERTY_IDENTIFIER = "identifier";
	public static final String ID_PROPERTY_RETURN_TYPE = "return type";
	public static final String ID_PROPERTY_VISIBILITY = "visibility";
	public static final String ID_PROPERTY_PARAMETERS = "parameters";
    private static final String ID_PROPERTY_VIEW_TYPE = "view-type";
	
	
	/**
	 * 
	 */
	public HomeMethodModel(Resource res)
	{
        factory.setTargetClass(MethodType.class);
        factory.setRes(res);
        method=(MethodType)factory.getNewObject();

		setSignature("public", "no_type", "no_identifier", new ArrayList());
		
		setViewType("LocalHome");
	}
	/**
	 * @param visibility
	 * @param returnType
	 * @param identifier
	 * @param parametersTypes
	 */
	public HomeMethodModel(Resource res,String visibility, String returnType, String identifier, List parametersTypes)
	{
        factory.setTargetClass(MethodType.class);
        factory.setRes(res);
        method=(MethodType)factory.getNewObject();
		setSignature(visibility, returnType, identifier, parametersTypes);
		
		setViewType("LocalHome");
	}
	
	/**
	 * @param visibility
	 * @param returnType
	 * @param identifier
	 * @param parameters
	 */
	public HomeMethodModel(Resource res,String visibility, String returnType, String identifier, String parameters)
    {
        factory.setTargetClass(MethodType.class);
        factory.setRes(res);
        method=(MethodType)factory.getNewObject();
        setSignature(visibility, returnType, identifier, parameters);
        
        setViewType("LocalHome");
    }
	
	/**
	 * @param visibility
	 * @param returnType
	 * @param identifier
	 * @param parametersTypes
	 */
	public void setSignature(String visibility, String returnType, String identifier, List parametersTypes)
	{
		this.visibility = visibility;
		this.returnType = returnType;
		this.identifier = identifier;
		this.parametersTypes = parametersTypes;
		firePropertyChange(PROPERTY_SIGNATURE_CHANGED, null, null);
	}
	
	/**
	 * @param visibility
	 * @param returnType
	 * @param identifier
	 * @param parameters
	 */
	public void setSignature(String visibility, String returnType, String identifier, String parameters)
    {
        this.visibility = visibility;
        this.returnType = returnType;
        this.identifier = identifier;
        this.parameters = parameters;
        parametersTypes = new ArrayList();
        parametersTypes.add(getParameters());
        firePropertyChange("signature", null, null);
    }
	
	/**
	 * @return
	 */
	public String getVisibility()
	{
		return visibility;
	}
	/**
	 * @return
	 */
	public String getReturnType()
	{
		return returnType;
	}
	/**
	 * @return
	 */
	public String getIdentifier()
	{
		return identifier;
	}
	/**
	 * @return
	 */
	public List getParametersTypes()
	{
		return parametersTypes;
	}
	
	/**
	 * @return
	 */
	public String getParameters()
    {
        return parameters;
    }

    /**
     * @param parameters
     */
    public void setParameters(String parameters)
    {
        String save = getParameters();
        this.parameters = parameters;
        getParametersTypes().clear();
        getParametersTypes().add(getParameters());
        firePropertyChange("signature", null, null);
    }
	
	/**
	 * @return
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		String txt = returnType + " " + identifier + "(";
		Iterator it = parametersTypes.iterator();
		if (it.hasNext())
			txt += it.next().toString();
		while (it.hasNext())
		{
			txt += " ," + it.next().toString();
		}
		txt += ")";
		return txt;
	}
	/**
	 * @param returnType
	 */
	public void setReturnType(String returnType)
	{
		this.returnType = returnType;
        getMethod().getMethodName().setId(returnType);
		firePropertyChange(PROPERTY_SIGNATURE_CHANGED, null, null);
	}
	/**
	 * @param identifier
	 */
	public void setIdentifier(String identifier)
	{
		this.identifier = identifier;
        getMethod().getMethodName().setValue(identifier);
		firePropertyChange(PROPERTY_SIGNATURE_CHANGED, null, null);
	}
	/**
	 * @param visibility
	 */
	public void setVisibility(String visibility)
	{
		if (visibility.equals("public") || visibility.equals("protected") || visibility.equals("private"))
		{
			this.visibility = visibility;
			
			firePropertyChange(PROPERTY_SIGNATURE_CHANGED, null, null);
		}
	}
	/**
	 * @return
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	public IPropertyDescriptor[] getPropertyDescriptors()
	{
		if (descriptors == null)
		{
		    descriptors = (new IPropertyDescriptor[] {

				new TextPropertyDescriptor(ID_PROPERTY_RETURN_TYPE, "return type"),
				new TextPropertyDescriptor(ID_PROPERTY_IDENTIFIER, "identifier"),
				new ComboBoxPropertyDescriptor(
					ID_PROPERTY_VISIBILITY,
					"visibility",
					new String[] { "public", "protected", "private" }),
                new TextPropertyDescriptor(ID_PROPERTY_PARAMETERS, "parameters"),
                new ComboBoxPropertyDescriptor(ID_PROPERTY_VIEW_TYPE, ID_PROPERTY_VIEW_TYPE,new String[]{"Home","LocalHome"})
		    });
		       
		}
		return descriptors;
	}
	/**
	 * @param propName
	 * @param val
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
	 */
	public void setPropertyValue(Object propName, Object val)
	{
		if (propName instanceof String)
		{
			if (((String) propName).equals(ID_PROPERTY_RETURN_TYPE))
				setReturnType(val.toString());
			else if (((String) propName).equals(ID_PROPERTY_IDENTIFIER))
				setIdentifier(val.toString());
            else if (((String) propName).equals(ID_PROPERTY_PARAMETERS))
                setParametersTypes(val.toString());
            else if (((String) propName).equals(ID_PROPERTY_VIEW_TYPE))
            {
                if(((Integer) val).intValue() == 0)
                    setViewType("Home");
                else
                    setViewType("LocalHome");
            }
              
			else if (((String) propName).equals(ID_PROPERTY_VISIBILITY))
			{
				if (((Integer) val).intValue() == 0)
					setVisibility(PUBLIC);
				else if (((Integer) val).intValue() == 2)
					setVisibility(PRIVATE);
				else if (((Integer) val).intValue() == 1)
					setVisibility(PROTECTED);
			}
		}
		else
			super.setPropertyValue(propName, val);
	}
	/**
	 * @param propName
	 * @return
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(Object propName)
	{
		if (propName instanceof String)
		{
			if (((String) propName).equals(ID_PROPERTY_RETURN_TYPE))
				return returnType;
			else if (((String) propName).equals(ID_PROPERTY_IDENTIFIER))
				return identifier;
            else if (((String) propName).equals(ID_PROPERTY_PARAMETERS))
                return showParams();
            else if (((String) propName).equals(ID_PROPERTY_VIEW_TYPE))
			{
				if (getViewType() == "Home")
					return new Integer(0);
				else 
				    return new Integer(1);
			}
			else if (((String) propName).equals(ID_PROPERTY_VISIBILITY))
			{
				if (visibility == PUBLIC)
					return new Integer(0);
				else if (visibility == PRIVATE)
					return new Integer(2);
				else if (visibility == PROTECTED)
					return new Integer(1);
				else
					return super.getPropertyValue(propName);
			}
			else
				return super.getPropertyValue(propName);
		}
		else
			return super.getPropertyValue(propName);
	}
    /**
     * @return Returns the method.
     */
    public MethodType getMethod() {
        return method;
    }
    /**
     * @param method The method to set.
     */
    public void setMethod(MethodType method) {
        this.method = method;
    }
    
    /**
     * @param parametersTypes The parametersTypes to set.
     */
    public void setParametersTypes(List parametersTypes) {
        this.parametersTypes = parametersTypes;
        firePropertyChange(PROPERTY_SIGNATURE_CHANGED, null, null);
    }
    
    public void setParametersTypes(String parameterList) {
        
        List params = new ArrayList();
        StringTokenizer token = new StringTokenizer(parameterList,",");
        method.getMethodParams().getMethodParam().clear();
        while(token.hasMoreTokens()) {
            
            StringTokenizer tokenParam = new StringTokenizer(token.nextToken()," ");
            String ident=tokenParam.nextToken();
            String retType=tokenParam.nextToken();
            
           
            // Model EMF instancie 
            factory.setTargetClass(JavaTypeType.class);
            JavaTypeType type = (JavaTypeType)factory.getNewObject();
          //  res.getContents().add(type);
            type.setId(retType);
            type.setValue(ident);
            method.getMethodParams().getMethodParam().add(type);
            
            params.add(ident+" "+retType);
        }
        
        setParametersTypes(params);

    }
    
    private String showParams(){
        List list = getParametersTypes();
        String params = "";
        Iterator it= list.iterator();
        while(it.hasNext()) {
            params += ((String)it.next());
            params +=",";
        }
        if(params.length() > 0)
            params=params.substring(0,params.length()-1);
        
        return params;
    }
    /**
     * @return Returns the viewType.
     */
    public String getViewType() {
        return viewType;
    }
    /**
     * @param viewType The viewType to set.
     */
    public void setViewType(String viewType) {
        this.viewType = viewType;
        
        getMethod().getMethodIntf().setValue(getViewType());
    }
}