package org.objectweb.jinha.testing.junit;

import java.util.List;
import junit.framework.TestCase;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.objectweb.jinha.generate.ProjectJINHa;
import org.objectweb.jinha.model.emf.jinha.GraphicalEntity;
import org.objectweb.jinha.model.emf.jinha.JinhaFactory;
import org.objectweb.jinha.model.emf.jinha.JinhaPackage;
import org.objectweb.jinha.model.emf.jinha.ModuleEjbDefinition;
import com.objectlearn.lomboz.xml.lomboz.DocumentRoot;
import com.objectlearn.lomboz.xml.lomboz.EjbDefinition;
import com.objectlearn.lomboz.xml.lomboz.EntityBean;
import com.objectlearn.lomboz.xml.lomboz.FieldMapping;
import com.objectlearn.lomboz.xml.lomboz.LombozFactory;
import com.objectlearn.lomboz.xml.lomboz.LombozPackage;
import com.objectlearn.lomboz.xml.lomboz.SessionBean;
import com.sun.java.xml.ns.j2ee.CmpFieldType;
import com.sun.java.xml.ns.j2ee.CmpVersionType;
import com.sun.java.xml.ns.j2ee.CmrFieldType;
import com.sun.java.xml.ns.j2ee.CmrFieldTypeType;
import com.sun.java.xml.ns.j2ee.EjbRelationType;
import com.sun.java.xml.ns.j2ee.EjbRelationshipRoleType;
import com.sun.java.xml.ns.j2ee.EntityBeanType;
import com.sun.java.xml.ns.j2ee.FullyQualifiedClassType;
import com.sun.java.xml.ns.j2ee.J2eeFactory;
import com.sun.java.xml.ns.j2ee.J2eePackage;
import com.sun.java.xml.ns.j2ee.JavaIdentifierType;
import com.sun.java.xml.ns.j2ee.MultiplicityType;
import com.sun.java.xml.ns.j2ee.PersistenceTypeType;
import com.sun.java.xml.ns.j2ee.RelationshipRoleSourceType;
import com.sun.java.xml.ns.j2ee.RelationshipsType;
import com.sun.java.xml.ns.j2ee.SessionBeanType;


/**
 * this class test an instance of an entity bean from the Lomboz EMF model;
 * it defines 2 entities ejb and the relationship between them 
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 *
  */
public class testEntityCreate extends TestCase {
    protected JinhaPackage jinhaPackage = JinhaPackage.eINSTANCE;

    protected JinhaFactory jinhaFactory = jinhaPackage.getJinhaFactory();

    protected J2eePackage j2eePackage = J2eePackage.eINSTANCE;

    protected J2eeFactory j2eeFactory = j2eePackage.getJ2eeFactory();

    protected LombozPackage lombozPackage = LombozPackage.eINSTANCE;

    protected LombozFactory lombozFactory = lombozPackage.getLombozFactory();
    
    protected ProjectJINHa project = new ProjectJINHa();
    
    IPackageFragmentRoot sourceFolder = null;

	
    /* 
     * @see junit.framework.TestCase#setUp()
     * @throws Exception
     */
    protected void setUp() throws Exception {
    	super.setUp();
    }
    
    /**
     * Create two entites ejb : entityA and entityB then the relationship
     * only entityA is initialized with the relation between lentityBeanA and lentityBeanB
     * so it's a monodirectionnal relation
     * @param module
     */
    public void TestRelation(ModuleEjbDefinition module){
    	
        GraphicalEntity result1 = jinhaFactory.createGraphicalEntity();
        GraphicalEntity result2 = jinhaFactory.createGraphicalEntity();
        
        // Cration des deux beans entits
    	EntityBean entityA = createEntity("ejbA","gg.ejbA");
        result1.setEntityBean(entityA);
    	EntityBean entityB = createEntity("ejbB","gg.ejbB");
        result2.setEntityBean(entityB);
        
        // Initialisation de la relation
    	RelationshipsType relationshipsType = createRelationshipsType(entityA,entityB);
    	entityA.setRelationships(relationshipsType);
    	
        module.getGraphicalEntityBeans().add(result1);
        module.getGraphicalEntityBeans().add(result2);
    }
    
    
    /**
     * Create an instance of an entity bean from EMF model,
     * it defines a general entity and the mapping to a database whith a CMP field 
     * @param ClassName : name of the classe, for example "classeA"
     * @param JNDIName : name of the package, it has to be "package.classeA" else generation does not function 
     * @return the EntityBean
     */
    public EntityBean createEntity(String ClassName,String JNDIName) {
		
		EntityBean lentityBean = lombozFactory.createEntityBean();
		EntityBeanType entityBean = j2eeFactory.createEntityBeanType();
				
		entityBean.setEjbName(j2eeFactory.createEjbNameType());
		entityBean.getEjbName().setValue(ClassName);
		entityBean.setEjbClass(j2eeFactory.createEjbClassType());
		entityBean.getEjbClass().setValue(JNDIName);
		
		// Persistence Type
		// * 	    Bean
		// * 	    Container
		PersistenceTypeType persistenceType = j2eeFactory.createPersistenceTypeType();
		persistenceType.setValue("Container");
		entityBean.setPersistenceType(persistenceType);
		lentityBean.setEntityEjb(entityBean);
		
		// Primary Key 
		com.sun.java.xml.ns.j2ee.String primaryKey = j2eeFactory.createString();
		primaryKey.setValue("id");
		entityBean.setPrimkeyField(primaryKey);
		FullyQualifiedClassType primaryKeyClass = j2eeFactory.createFullyQualifiedClassType();
		primaryKeyClass.setValue("java.lang.String");
		entityBean.setPrimKeyClass(primaryKeyClass);
		
		// Schema 
		JavaIdentifierType schema = j2eeFactory.createJavaIdentifierType();
		schema.setValue("MYSCHEMA");
		entityBean.setAbstractSchemaName(schema);
		
		// CMP Version 
		//		 * 	    1.x
		//		 * 	    2.x
		CmpVersionType cmpVersion = j2eeFactory.createCmpVersionType();
		cmpVersion.setValue("2.x");
		entityBean.setCmpVersion(cmpVersion);
		
		
		// CMP Fields ID
		CmpFieldType cmpField = j2eeFactory.createCmpFieldType();
		JavaIdentifierType fieldName = j2eeFactory.createJavaIdentifierType();
		fieldName.setValue("id");
		JavaIdentifierType fieldName2 = j2eeFactory.createJavaIdentifierType();
		fieldName2.setValue("id");
		FullyQualifiedClassType fieldType = j2eeFactory.createFullyQualifiedClassType();
		fieldType.setValue("java.lang.String");
		
		JavaIdentifierType columnName = j2eeFactory.createJavaIdentifierType();
		columnName.setValue("ID_FLD");
		//		ARRAY BIGINT BINARY	BIT	BLOB BOOLEAN CHAR CLOB	DATALINK
		//		DATE DECIMAL DISTINCT DOUBLE FLOAT INTEGER JAVA_OBJECT LONGVARBINARY
		//		LONGVARCHAR	NULL NUMERIC OTHER REAL	REF SMALLINT STRUCT TIME TIMESTAMP
		//		TINYINT	VARBINARY VARCHAR
		JavaIdentifierType jdbcType = j2eeFactory.createJavaIdentifierType();
		jdbcType.setValue("VARCHAR");
		JavaIdentifierType sqlType = j2eeFactory.createJavaIdentifierType();
		sqlType.setValue("VARCHAR(100)");
		
		cmpField.setFieldName(fieldName2);
		FieldMapping fieldMapping = lombozFactory.createFieldMapping();
		fieldMapping.setReadOnly(false);
		fieldMapping.setPrimaryKey(true);
		fieldMapping.setFieldName(fieldName);
		fieldMapping.setFieldType(fieldType);
		fieldMapping.setColumnName(columnName);
		fieldMapping.setJdbcType(jdbcType);
		fieldMapping.setSqlType(sqlType);	
		entityBean.getCmpField().add(cmpField);
		lentityBean.getFieldMappings().add(fieldMapping);
	
		// CMP Fields ADDRESS
		cmpField = j2eeFactory.createCmpFieldType();
		fieldName = j2eeFactory.createJavaIdentifierType();
		fieldName.setValue("address");
		fieldName2 = j2eeFactory.createJavaIdentifierType();
		fieldName2.setValue("address");
		fieldType = j2eeFactory.createFullyQualifiedClassType();
		fieldType.setValue("java.lang.String");
		columnName = j2eeFactory.createJavaIdentifierType();
		columnName.setValue("ADDRESS_FLD");
		//		ARRAY BIGINT BINARY	BIT	BLOB BOOLEAN CHAR CLOB	DATALINK
		//		DATE DECIMAL DISTINCT DOUBLE FLOAT INTEGER JAVA_OBJECT LONGVARBINARY
		//		LONGVARCHAR	NULL NUMERIC OTHER REAL	REF SMALLINT STRUCT TIME TIMESTAMP
		//		TINYINT	VARBINARY VARCHAR
		jdbcType = j2eeFactory.createJavaIdentifierType();
		jdbcType.setValue("VARCHAR");
		sqlType = j2eeFactory.createJavaIdentifierType();
		sqlType.setValue("VARCHAR(100)");
		
		cmpField.setFieldName(fieldName2);
		fieldMapping = lombozFactory.createFieldMapping();
		fieldMapping.setReadOnly(false);
		fieldMapping.setPrimaryKey(false);
		fieldMapping.setFieldName(fieldName);
		fieldMapping.setFieldType(fieldType);
		fieldMapping.setColumnName(columnName);
		fieldMapping.setJdbcType(jdbcType);
		fieldMapping.setSqlType(sqlType);	
		entityBean.getCmpField().add(cmpField);
		lentityBean.getFieldMappings().add(fieldMapping);
	
		// CMP Fields count
		cmpField = j2eeFactory.createCmpFieldType();
		fieldName = j2eeFactory.createJavaIdentifierType();
		fieldName.setValue("count");
		fieldName2 = j2eeFactory.createJavaIdentifierType();
		fieldName2.setValue("count");
		fieldType = j2eeFactory.createFullyQualifiedClassType();
		fieldType.setValue("java.lang.Integer");
		columnName = j2eeFactory.createJavaIdentifierType();
		columnName.setValue("ACNTR_FLD");
		//		ARRAY BIGINT BINARY	BIT	BLOB BOOLEAN CHAR CLOB	DATALINK
		//		DATE DECIMAL DISTINCT DOUBLE FLOAT INTEGER JAVA_OBJECT LONGVARBINARY
		//		LONGVARCHAR	NULL NUMERIC OTHER REAL	REF SMALLINT STRUCT TIME TIMESTAMP
		//		TINYINT	VARBINARY VARCHAR
		jdbcType = j2eeFactory.createJavaIdentifierType();
		jdbcType.setValue("INTEGER");
		sqlType = j2eeFactory.createJavaIdentifierType();
		sqlType.setValue("INTEGER");
		
		cmpField.setFieldName(fieldName2);
		fieldMapping = lombozFactory.createFieldMapping();
		fieldMapping.setReadOnly(false);
		fieldMapping.setPrimaryKey(false);
		fieldMapping.setFieldName(fieldName);
		fieldMapping.setFieldType(fieldType);
		fieldMapping.setColumnName(columnName);
		fieldMapping.setJdbcType(jdbcType);
		fieldMapping.setSqlType(sqlType);	
		entityBean.getCmpField().add(cmpField);
		lentityBean.getFieldMappings().add(fieldMapping);
		
		
		lentityBean.setTableName("MY_CMP_BEAN_TBL");
		lentityBean.setDataSourceName("MY_JDBC_DATASOURCE");
				
		return lentityBean;
    }
    
		
    /**
     * Create an instance of a relationship one-to-many from EMF model between the 2 entities ejb in parameters
     * @param lentityBeanA : this entity is initialized with the relation between lentityBeanA and lentityBeanB
     * @param lentityBeanB : the other bean of the relation
     * @return the Relationshipstype
     */
    public RelationshipsType createRelationshipsType(EntityBean lentityBeanA, EntityBean lentityBeanB) {
    	
    	RelationshipsType relationshipsType = j2eeFactory.createRelationshipsType();
    	EjbRelationType ejbRelationType = j2eeFactory.createEjbRelationType();
		lentityBeanA.setRelationships(relationshipsType);
		relationshipsType.getEjbRelation().add(ejbRelationType);
		
		// Initialisation de la relation avec l'objet EjbRelationType
		com.sun.java.xml.ns.j2ee.String relationName = j2eeFactory.createString();
		relationName.setValue(lentityBeanA.getEntityEjb().getEjbName().getValue()+"-"+lentityBeanA.getEntityEjb().getEjbName().getValue()); 
		EjbRelationshipRoleType ejbRelationshipRole1 = j2eeFactory.createEjbRelationshipRoleType(); 
		EjbRelationshipRoleType ejbRelationshipRole2 = j2eeFactory.createEjbRelationshipRoleType();
		ejbRelationType.setEjbRelationName(relationName);
		ejbRelationType.setEjbRelationshipRole(ejbRelationshipRole1);
		ejbRelationType.setEjbRelationshipRole(ejbRelationshipRole2);
		
		// Initialisation de ejbRelationshipRole1
		com.sun.java.xml.ns.j2ee.String shipRelationName = j2eeFactory.createString();
		shipRelationName.setValue(lentityBeanB.getEntityEjb().getEjbName().getValue()+"-HAS-"+lentityBeanA.getEntityEjb().getEjbName().getValue());
		ejbRelationshipRole1.setEjbRelationshipRoleName(shipRelationName);
		MultiplicityType multiplicityType = j2eeFactory.createMultiplicityType();
		multiplicityType.setValue("One");
		ejbRelationshipRole1.setMultiplicity(multiplicityType);
		
		// TODO ajouter relationshipRoleSourceType si ncessaire (ne semble pas obligatoire) 
		RelationshipRoleSourceType relationshipRoleSourceType = j2eeFactory.createRelationshipRoleSourceType();
		
		// Cration champ CMR
		CmrFieldType cmrFieldType = j2eeFactory.createCmrFieldType();
		com.sun.java.xml.ns.j2ee.String cmrFieldName = j2eeFactory.createString();
		cmrFieldName.setValue(lentityBeanB.getEntityEjb().getEjbName().getValue());
		cmrFieldType.setCmrFieldName(cmrFieldName);
		CmrFieldTypeType cmrFieldTypeType = j2eeFactory.createCmrFieldTypeType();
		cmrFieldTypeType.setValue("java.util.Set");
		cmrFieldType.setCmrFieldType(cmrFieldTypeType);
		ejbRelationshipRole1.setCmrField(cmrFieldType);	
		
		// Initialisation de ejbRelationshipRole2
		com.sun.java.xml.ns.j2ee.String shipRelationName_ = j2eeFactory.createString();
		shipRelationName_.setValue(lentityBeanA.getEntityEjb().getEjbName().getValue()+"-HAS-"+lentityBeanB.getEntityEjb().getEjbName().getValue());
		ejbRelationshipRole2.setEjbRelationshipRoleName(shipRelationName_);
		MultiplicityType multiplicityType_ = j2eeFactory.createMultiplicityType();
		multiplicityType_.setValue("Many");
		ejbRelationshipRole2.setMultiplicity(multiplicityType_);
		
		// TODO ajouter relationshipRoleSourceType si ncessaire (ne semble pas obligatoire) 
		RelationshipRoleSourceType relationshipRoleSourceType_ = j2eeFactory.createRelationshipRoleSourceType();
		
		// Cration champ CMR
		CmrFieldType cmrFieldType_ = j2eeFactory.createCmrFieldType();
		com.sun.java.xml.ns.j2ee.String cmrFieldName_ = j2eeFactory.createString();
		cmrFieldName_.setValue(lentityBeanA.getEntityEjb().getEjbName().getValue());
		cmrFieldType.setCmrFieldName(cmrFieldName_);
		CmrFieldTypeType cmrFieldTypeType_ = j2eeFactory.createCmrFieldTypeType();
		cmrFieldTypeType.setValue("java.util.Collection");
		cmrFieldType_.setCmrFieldType(cmrFieldTypeType_);
		ejbRelationshipRole2.setCmrField(cmrFieldType_);	
		
		return relationshipsType;
	}
}
