package org.objectweb.jinha.testing.junit;

import junit.framework.TestCase;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.objectweb.jinha.generate.ProjectJINHa;
import org.objectweb.jinha.model.emf.jinha.GraphicalSession;
import org.objectweb.jinha.model.emf.jinha.JinhaFactory;
import org.objectweb.jinha.model.emf.jinha.JinhaPackage;
import com.objectlearn.lomboz.xml.lomboz.Attribute;
import com.objectlearn.lomboz.xml.lomboz.LombozFactory;
import com.objectlearn.lomboz.xml.lomboz.LombozPackage;
import com.objectlearn.lomboz.xml.lomboz.SessionBean;
import com.sun.java.xml.ns.j2ee.J2eeFactory;
import com.sun.java.xml.ns.j2ee.J2eePackage;
import com.sun.java.xml.ns.j2ee.JavaIdentifierType;
import com.sun.java.xml.ns.j2ee.JavaTypeType;
import com.sun.java.xml.ns.j2ee.MethodIntfType;
import com.sun.java.xml.ns.j2ee.MethodNameType;
import com.sun.java.xml.ns.j2ee.MethodParamsType;
import com.sun.java.xml.ns.j2ee.MethodType;
import com.sun.java.xml.ns.j2ee.SessionBeanType;


/**
 * this class test an instance of an entity bean from the Lomboz EMF model
 * @author LAFON Jerome, CRESSON Sylvain, PIGNARD Quentin
 */
public class testSessionCreate extends TestCase {
    protected JinhaPackage jinhaPackage = JinhaPackage.eINSTANCE;

    protected JinhaFactory jinhaFactory = jinhaPackage.getJinhaFactory();

    protected J2eePackage j2eePackage = J2eePackage.eINSTANCE;

    protected J2eeFactory j2eeFactory = j2eePackage.getJ2eeFactory();

    protected LombozPackage lombozPackage = LombozPackage.eINSTANCE;

    protected LombozFactory lombozFactory = lombozPackage.getLombozFactory();
    
    protected ProjectJINHa project = new ProjectJINHa();
    
    IPackageFragmentRoot sourceFolder = null;

	/*
	 * @see TestCase#setUp()
	 */
    protected void setUp() throws Exception {
    	super.setUp();
    }
        
    /**
     * Create an instance of a session bean
     * @param ClassName : name of the classe, for example "classeA"
     * @param JNDIName : name of the package, it has to be "package.classeA" else generation does not function
     * @return an instance of a SessionBean
     */
    public SessionBean createSessionEJB(String className, String jndiName) {

        SessionBean lsessionBean = lombozFactory.createSessionBean();
        SessionBeanType sessionBean = j2eeFactory.createSessionBeanType();
        lsessionBean.setSessionEjb(sessionBean);
        sessionBean.setEjbName(j2eeFactory.createEjbNameType());
        sessionBean.getEjbName().setValue(className); 
        sessionBean.setEjbClass(j2eeFactory.createEjbClassType());
        sessionBean.getEjbClass().setValue(jndiName);
        sessionBean.setSessionType(j2eeFactory.createSessionTypeType());
        sessionBean.getSessionType().setValue("Stateful");
        sessionBean.setTransactionType(j2eeFactory.createTransactionTypeType());
        sessionBean.getTransactionType().setValue("transaction1");
        
        return lsessionBean;
    }
    
        
    /**
     * Create a method with the parameters provided
     * @param accessibility : private, public or protected
     * @param returnType : type of return of the method
     * @param name : name of the method
     * @param params : parameters of the method , table of name and type
     * @return the method instancied
     */
    private MethodType CreateSessionBeanMethods(String accessibility,
                                                String returnType,
                                                String name,
                                                String[][] params/*name,type*/) {
               

        MethodType method = j2eeFactory.createMethodType();
        MethodParamsType methodParamsType = j2eeFactory.createMethodParamsType();
        method.setMethodParams(methodParamsType);
        MethodNameType methodName = j2eeFactory.createMethodNameType();
        MethodIntfType intfType = j2eeFactory.createMethodIntfType();
        intfType.setValue(accessibility);        
        method.setMethodIntf(intfType);     
        methodName.setId(returnType);
        methodName.setValue(name);
        method.setMethodName(methodName);
                 
        for(int i=0;i<params.length;i++) {
            JavaTypeType type = j2eeFactory.createJavaTypeType();
            type.setId(params[i][1]);
            type.setValue(params[i][0]);
            methodParamsType.getMethodParam().add(type);
       }
       return method;
    }

    /**
     * Create an Attribut from the model 
     * @param visibility : private, public or protected
     * @param type : type of the attribut
     * @param name : name of the attribut
     * @return the Attribut
     */
    private Attribute createJavaAttribut(String visibility,String type,String name) {
                
            Attribute attribute1 = lombozFactory.createAttribute();
            JavaTypeType javatypetype = j2eeFactory.createJavaTypeType();
            javatypetype.setValue(type);
            JavaIdentifierType javaIdentifierType = j2eeFactory.createJavaIdentifierType(); 
            javaIdentifierType.setValue(name);
            attribute1.setName(javaIdentifierType);
            attribute1.setType(javatypetype);
            com.sun.java.xml.ns.j2ee.String Visibility = j2eeFactory.createString();
            Visibility.setValue(visibility);
            attribute1.setVisibility(Visibility);
            return attribute1;
     }
    
    /**
     * Creation of a GraphicalSession (an instance of the JINHa EMF model) 
     * We have define some obligatories methods and attributes from the spec J2ee
     * @param className : used by createSessionEJB
     * @param jndiName : used by createSessionEJB
     * @return the GraphicalSession
     */
    public GraphicalSession createGraphicalSession(String className, String jndiName){
    	
        GraphicalSession result = jinhaFactory.createGraphicalSession();
        SessionBean sessionBean=lombozFactory.createSessionBean();
        SessionBeanType sessionBeanType=j2eeFactory.createSessionBeanType();
        sessionBean.setSessionEjb(sessionBeanType);
        
        // cration du bean
        sessionBean = createSessionEJB(className,jndiName);
              
        // Cration des 2 mthodes
        MethodType methode1 = CreateSessionBeanMethods("Home","void","methodHome",new String[][] {{"int","param1"},{"String","param2"}});
        MethodType methode2 = CreateSessionBeanMethods("Remote","void","methodBusiness",new String[][] {{"String","param1"},{"float","param2"}});
                                  
        // Cration de l'attribut
        Attribute attr1=createJavaAttribut("public","int","var1");
                
        // Cration des mthodes Home obligatoires aux beans session
        Attribute sessionctx    = createJavaAttribut("protected","SessionContext","sessionContext");
        MethodType methode3     = CreateSessionBeanMethods("Home","void","ejbcreate",new String[0][0]);
        MethodType methode4     = CreateSessionBeanMethods("Home","void","ejbActivate",new String[0][0]);
        MethodType methode5     = CreateSessionBeanMethods("Home","void","ejbPassivate",new String[0][0]);
        MethodType methode6     = CreateSessionBeanMethods("Home","void","ejbRemove",new String[0][0]);
        MethodType methode7     = CreateSessionBeanMethods("Home","void","setSessionContext",new String[][] {{"SessionContext","sessionContext"}});
        
        // Ajout des mthodes et attributs au bean
        sessionBean.getJavaAttributes().add(attr1);
        sessionBean.getJavaAttributes().add(sessionctx);
        sessionBean.getMethods().add(methode1);
        sessionBean.getMethods().add(methode2);
        sessionBean.getMethods().add(methode3);
        sessionBean.getMethods().add(methode4);
        sessionBean.getMethods().add(methode5);
        sessionBean.getMethods().add(methode6);
        sessionBean.getMethods().add(methode7);
        
        ((GraphicalSession)result).setSessionBean(sessionBean);
        
        return result;
    }
}
